#----------------------------------------------------------------------------
# Name:         printout.py
# Purpose:      preview and printing class -> table/grid printing
#
# Author:       Lorne White (email: lorne.white@telusplanet.net)
#
# Created:
# Version       0.75
# Date:         May 15, 2002
# Licence:      wxWindows license
#----------------------------------------------------------------------------
# Release Notes

# fixed bug for string wider than print region
# add index to data list after parsing total pages for paging
#----------------------------------------------------------------------------

import os, sys, copy

from wxPython.wx import *
import copy

class PrintBase:
    def SetPrintFont(self, font):      # set the DC font parameters
        fattr = font["Attr"]
        if fattr[0] == 1:
            weight = wxBOLD
        else:
            weight = wxNORMAL

        if fattr[1] == 1:
            set_style = wxITALIC
        else:
            set_style = wxNORMAL

        underline = fattr[2]
        fcolour = self.GetFontColour(font)
        self.DC.SetTextForeground(fcolour)

        setfont = wxFont(font["Size"], wxSWISS, set_style, weight, underline)
        setfont.SetFaceName(font["Name"])
        self.DC.SetFont(setfont)

    def GetFontColour(self, font):
        fcolour = font["Colour"]
        return wxColour(fcolour[0], fcolour[1], fcolour[2])

    def OutTextRegion(self, textout, txtdraw = True):
        textlines = textout.split('\n')
        y = copy.copy(self.y) + self.pt_space_before
        for text in textlines:
            remain = 'X'
            while remain != "":
                vout, remain = self.SetFlow(text, self.region)
                if self.draw == True and txtdraw == True:
                    test_out = self.TestFull(vout)
                    if self.align == wxALIGN_LEFT:
                        self.DC.DrawText(test_out, self.indent+self.pcell_left_margin, y)

                    elif self.align == wxALIGN_CENTRE:
                        diff = self.GetCellDiff(test_out, self.region)
                        self.DC.DrawText(test_out, self.indent+diff/2, y)

                    elif self.align == wxALIGN_RIGHT:
                        diff = self.GetCellDiff(test_out, self.region)
                        self.DC.DrawText(test_out, self.indent+diff, y)

                    else:
                        self.DC.DrawText(test_out, self.indent+self.pcell_left_margin, y)
                text = remain
                y = y + self.space
        return y - self.space + self.pt_space_after

    def GetCellDiff(self, text, width):      # get the remaining cell size for adjustment
        w, h = self.DC.GetTextExtent(text)
        diff = width - w
        if diff < 0:
            diff = 0
        return diff

    def TestFull(self, text_test):
        w, h = self.DC.GetTextExtent(text_test)
        if w > self.region:     # trouble fitting into cell
            return self.SetChar(text_test, self.region)     # fit the text to the cell size
        else:
            return text_test

    def SetFlow(self, ln_text, width):
        width = width - self.pcell_right_margin
        text = ""
        split = ln_text.split()
        if len(split) == 1:
            return ln_text, ""

        try:
            w, h = self.DC.GetTextExtent(" " + split[0])
            if w >= width:
                return ln_text, ""
        except:
            pass

        cnt = 0
        for word in split:
            bword = " " + word  # blank + word
            length = len(bword)

            w, h = self.DC.GetTextExtent(text + bword)
            if w < width:
                text = text + bword
                cnt = cnt + 1
            else:
                remain = ' '.join(split[cnt:])
                text = text.strip()
                return text, remain

        remain = ' '.join(split[cnt:])
        vout = text.strip()
        return vout, remain

    def SetChar(self, ln_text, width):  # truncate string to fit into width
        width = width - self.pcell_right_margin - self.pcell_left_margin
        text = ""
        for val in ln_text:
            w, h = self.DC.GetTextExtent(text + val)
            if w > width:
                text = text + ".."
                return text     # fitted text value
            text = text + val
        return text

    def OutTextPageWidth(self, textout, y_out, align, indent, txtdraw = True):
        textlines = textout.split('\n')
        y = copy.copy(y_out)

        pagew = self.parent.page_width * self.pwidth        # full page width
        w, h = self.DC.GetTextExtent(textout)
        y_line = h

        for text in textlines:
            remain = 'X'
            while remain != "":
                vout, remain = self.SetFlow(text, pagew)
                if self.draw == True and txtdraw == True:
                    test_out = vout
                    if align == wxALIGN_LEFT:
                        self.DC.DrawText(test_out, indent, y)

                    elif align == wxALIGN_CENTRE:
                        diff = self.GetCellDiff(test_out, pagew)
                        self.DC.DrawText(test_out, indent+diff/2, y)

                    elif align == wxALIGN_RIGHT:
                        diff = self.GetCellDiff(test_out, pagew)
                        self.DC.DrawText(test_out, indent+diff, y)

                    else:
                        self.DC.DrawText(test_out, indent, y_out)
                text = remain
                y = y + y_line
        return y - y_line

    def GetDate(self):
        date, time = self.GetNow()
        return date

    def GetDateTime(self):
        date, time = self.GetNow()
        return date + ' ' + time

    def GetNow(self):
        full = str(wxDateTime_Now())        # get the current date and time in print format
        flds = full.split()
        date = flds[0]
        time = flds[1]
        return date, time

    def SetPreview(self, preview):
        self.preview = preview

    def SetPSize(self, width, height):
        self.pwidth = width/self.scale
        self.pheight = height/self.scale

    def SetScale(self, scale):
        self.scale = scale

    def SetPTSize(self, width, height):
        self.ptwidth = width
        self.ptheight = height

    def getWidth(self):
        return self.sizew

    def getHeight(self):
        return self.sizeh


class PrintTableDraw(wxScrolledWindow, PrintBase):
    def __init__(self, parent, DC, size):
        self.parent = parent
        self.DC = DC
        self.scale = parent.scale
        self.width = size[0]
        self.height = size[1]
        self.SetDefaults()

    def SetDefaults(self):
        self.page = 1
        self.total_pages = None

        self.page_width = self.parent.page_width
        self.page_height = self.parent.page_height

        self.left_margin = self.parent.left_margin
        self.right_margin = self.parent.right_margin

        self.top_margin  = self.parent.top_margin
        self.bottom_margin = self.parent.bottom_margin
        self.cell_left_margin = self.parent.cell_left_margin
        self.cell_right_margin = self.parent.cell_right_margin

        self.label_colour = self.parent.label_colour

        self.row_line_colour = self.parent.row_line_colour
        self.row_line_size = self.parent.row_line_size

        self.row_def_line_colour = self.parent.row_def_line_colour
        self.row_def_line_size = self.parent.row_def_line_size

        self.column_line_colour = self.parent.column_line_colour
        self.column_line_size = self.parent.column_line_size

        self.column_def_line_size = self.parent.column_def_line_size
        self.column_def_line_colour = self.parent.column_def_line_colour

        self.text_font = self.parent.text_font

        self.label_font = self.parent.label_font

    def AdjustValues(self):
        self.vertical_offset = self.pheight * self.parent.vertical_offset
        self.horizontal_offset = self.pheight * self.parent.horizontal_offset

        self.pcell_left_margin = self.pwidth * self.cell_left_margin
        self.pcell_right_margin = self.pwidth * self.cell_right_margin
        self.ptop_margin = self.pheight * self.top_margin
        self.pbottom_margin = self.pheight * self.bottom_margin

        self.pheader_margin = self.pheight * self.parent.header_margin
        self.pfooter_margin = self.pheight * self.parent.footer_margin

        self.cell_colour = self.parent.set_cell_colour
        self.cell_text = self.parent.set_cell_text

        self.column = []
        self.column_align = []
        self.column_bgcolour = []
        self.column_txtcolour = []

        set_column_align = self.parent.set_column_align
        set_column_bgcolour = self.parent.set_column_bgcolour
        set_column_txtcolour = self.parent.set_column_txtcolour

        pos_x = self.left_margin * self.pwidth + self.horizontal_offset     # left margin
        self.column.append(pos_x)

        if self.set_column == []:
            table_width = self.page_width - self.left_margin - self.right_margin
            width = table_width/(len(self.label))
            for val in self.label:
                column_width = width * self.pwidth
                pos_x = pos_x + column_width
                self.column.append(pos_x)   # position of each column
        else:
            for val in self.set_column:
                column_width = val * self.pwidth
                pos_x = pos_x + column_width
                self.column.append(pos_x)   # position of each column

        if pos_x > self.page_width * self.pwidth:    # check if it fits in page
            print "Warning, Too Wide for Page"
            return

        if self.label != []:
            if len(self.column) -1 != len(self.label):
                print "Column Settings Incorrect", "\nColumn Value: " + str(self.column), "\nLabel Value: " + str(self.label)
                return

        first_value = self.data[0]
        column_total = len(first_value)
        if column_total != len(self.column) -1:
            print "Column Settings Incorrect", first_value, self.column
            return

        col = 0
        for col in range(column_total):
            try:
                align = set_column_align[col]       # check if custom column alignment
            except:
                align = wxALIGN_LEFT
            self.column_align.append(align)

            try:
                colour = set_column_bgcolour[col]     # check if custom column background colour
            except:
                colour = self.parent.column_colour
            self.column_bgcolour.append(colour)

            try:
                colour = set_column_txtcolour[col]     # check if custom column text colour
            except:
                colour = self.GetFontColour(self.parent.text_font)

            self.column_txtcolour.append(colour)

            col = col + 1

    def SetPointAdjust(self):
        f = wxFont(10, wxSWISS, wxNORMAL, wxNORMAL)     # setup using 10 point
        self.DC.SetFont(f)
        f.SetFaceName(self.text_font["Name"])
        x, y = self.DC.GetTextExtent("W")

        self.label_pt_space_before = self.parent.label_pt_adj_before * y/10        # extra spacing for label per point value
        self.label_pt_space_after = self.parent.label_pt_adj_after * y/10

        self.text_pt_space_before = self.parent.text_pt_adj_before * y/10        # extra spacing for row text per point value
        self.text_pt_space_after = self.parent.text_pt_adj_after * y/10

    def SetPage(self, page):
        self.page = page

    def SetColumns(self, col):
        self.column = col

    def OutCanvas(self):
        self.AdjustValues()
        self.SetPointAdjust()

        self.y_start = self.ptop_margin + self.vertical_offset
        self.y_end = self.parent.page_height * self.pheight - self.pbottom_margin + self.vertical_offset

        self.SetPrintFont(self.label_font)

        x, y = self.DC.GetTextExtent("W")
        self.label_space = y

        self.SetPrintFont(self.text_font)

        x, y = self.DC.GetTextExtent("W")
        self.space = y

        if self.total_pages is None:
            self.GetTotalPages()    # total pages for display/printing

        self.data_cnt = self.page_index[self.page-1]

        self.draw = True
        self.PrintHeader()
        self.PrintFooter()
        self.OutPage()

    def GetTotalPages(self):
        self.data_cnt = 0
        self.draw = False
        self.page_index = [0]

        cnt = 0
        while 1:
            test = self.OutPage()
            self.page_index.append(self.data_cnt)
            if  test == False:
                break
            cnt = cnt + 1

        self.total_pages = cnt + 1

    def OutPage(self):
        self.y = self.y_start
        self.end_x = self.column[-1]

        if self.data_cnt < len(self.data):  # if there data for display on the page
            if self.label != []:        # check if header defined
                self.PrintLabel()
        else:
            return False

        for val in self.data:
            try:
                row_val = self.data[self.data_cnt]
            except:
                self.FinishDraw()
                return False

            max_y = self.PrintRow(row_val, False)       # test to see if row will fit in remaining space
            test = max_y + self.space

            if test > self.y_end:
                break

            self.ColourRowCells(max_y-self.y+self.space)       # colour the row/column
            max_y = self.PrintRow(row_val, True)      # row fits - print text
            self.DrawGridLine()     # top line of cell
            self.y = max_y + self.space

            if self.y > self.y_end:
                break

            self.data_cnt = self.data_cnt + 1

        self.FinishDraw()

        if self.data_cnt == len(self.data):    # last value in list
            return False

        return True


    def PrintLabel(self):
        self.pt_space_before = self.label_pt_space_before   # set the point spacing
        self.pt_space_after = self.label_pt_space_after

        self.LabelColorRow(self.label_colour)
        self.SetPrintFont(self.label_font)

        self.col = 0
        max_y = 0
        for vtxt in self.label:
            self.region = self.column[self.col+1] - self.column[self.col]
            self.indent = self.column[self.col]

            self.align = wxALIGN_LEFT

            max_out = self.OutTextRegion(vtxt, True)
            if max_out > max_y:
                max_y = max_out
            self.col = self.col + 1

        self.DrawGridLine()     # top line of label
        self.y = max_y + self.label_space

    def PrintHeader(self):      # print the header array
        if self.draw == False:
            return

        for val in self.parent.header:
            self.SetPrintFont(val["Font"])

            header_indent = val["Indent"] * self.pwidth
            text = val["Text"]

            htype = val["Type"]
            if htype == "Date":
                addtext = self.GetDate()
            elif htype == "Date & Time":
                addtext = self.GetDateTime()
            else:
                addtext = ""

            self.OutTextPageWidth(text+addtext, self.pheader_margin, val["Align"], header_indent, True)

    def PrintFooter(self):      # print the header array
        if self.draw == False:
            return

        footer_pos = self.parent.page_height * self.pheight - self.pfooter_margin + self.vertical_offset
        for val in self.parent.footer:
            self.SetPrintFont(val["Font"])

            footer_indent = val["Indent"] * self.pwidth
            text = val["Text"]

            ftype = val["Type"]
            if ftype == "Pageof":
                addtext = "Page " + str(self.page) + " of " + str(self.total_pages)
            elif ftype == "Page":
                addtext = "Page " + str(self.page)
            elif ftype == "Num":
                addtext = str(self.page)
            elif ftype == "Date":
                addtext = self.GetDate()
            elif ftype == "Date & Time":
                addtext = self.GetDateTime()
            else:
                addtext = ""

            self.OutTextPageWidth(text+addtext, footer_pos, val["Align"], footer_indent, True)


    def LabelColorRow(self, colour):
        brush = wxBrush(colour, wxSOLID)
        self.DC.SetBrush(brush)
        height = self.label_space + self.label_pt_space_before + self.label_pt_space_after
        self.DC.DrawRectangle(self.column[0], self.y, self.end_x-self.column[0]+1, height)

    def ColourRowCells(self, height):
        if self.draw == False:
            return

        col = 0
        for colour in self.column_bgcolour:
            cellcolour = self.GetCellColour(self.data_cnt, col)
            if cellcolour is not None:
                colour = cellcolour

            brush = wxBrush(colour, wxSOLID)
            self.DC.SetBrush(brush)
            self.DC.SetPen(wxPen(wxNamedColour('WHITE'), 0))

            start_x = self.column[col]
            width = self.column[col+1] - start_x + 2
            self.DC.DrawRectangle(start_x, self.y, width, height)
            col = col + 1

    def PrintRow(self, row_val, draw = True, align = wxALIGN_LEFT):
        self.SetPrintFont(self.text_font)

        self.pt_space_before = self.text_pt_space_before   # set the point spacing
        self.pt_space_after = self.text_pt_space_after

        self.col = 0
        max_y = 0
        for vtxt in row_val:
            self.region = self.column[self.col+1] - self.column[self.col]
            self.indent = self.column[self.col]
            self.align = self.column_align[self.col]

            fcolour = self.column_txtcolour[self.col]       # set font colour
            celltext = self.GetCellText(self.data_cnt, self.col)
            if celltext is not None:
                fcolour = celltext      # override the column colour

            self.DC.SetTextForeground(fcolour)

            max_out = self.OutTextRegion(vtxt, draw)
            if max_out > max_y:
                max_y = max_out
            self.col = self.col + 1
        return max_y

    def GetCellColour(self, row, col):      # check if custom colour defined for the cell background
        try:
            set = self.cell_colour[row]
        except:
            return None
        try:
            colour = set[col]
            return colour
        except:
            return None

    def GetCellText(self, row, col):      # check if custom colour defined for the cell text
        try:
            set = self.cell_text[row]
        except:
            return None
        try:
            colour = set[col]
            return colour
        except:
            return None

    def FinishDraw(self):
        self.DrawGridLine()     # draw last row line
        self.DrawColumns()      # draw all vertical lines

    def DrawGridLine(self):
        if self.draw == True:
            try:
                size = self.row_line_size[self.data_cnt]
            except:
                size = self.row_def_line_size

            try:
                colour = self.row_line_colour[self.data_cnt]
            except:
                colour = self.row_def_line_colour

            self.DC.SetPen(wxPen(colour, size))

            y_out = self.y
#            y_out = self.y + self.pt_space_before + self.pt_space_after     # adjust for extra spacing
            self.DC.DrawLine(self.column[0], y_out, self.end_x, y_out)

    def DrawColumns(self):
        if self.draw == True:
            col = 0
            for val in self.column:
                try:
                    size = self.column_line_size[col]
                except:
                    size = self.column_def_line_size

                try:
                    colour = self.column_line_colour[col]
                except:
                    colour = self.column_def_line_colour

                indent = val

                self.DC.SetPen(wxPen(colour, size))
                self.DC.DrawLine(indent, self.y_start, indent, self.y)
                col = col + 1

    def DrawText(self):
        self.DoRefresh()

    def DoDrawing(self, DC):
        size = DC.GetSizeTuple()
        self.DC = DC

        DC.BeginDrawing()
        self.DrawText()
        DC.EndDrawing()

        self.sizew = DC.MaxY()
        self.sizeh = DC.MaxX()


class PrintTable:
    def __init__(self, parentFrame=None):
        self.data = []
        self.set_column = []
        self.label = []
        self.header = []
        self.footer = []

        self.set_column_align = {}
        self.set_column_bgcolour = {}
        self.set_column_txtcolour = {}
        self.set_cell_colour = {}
        self.set_cell_text = {}
        self.column_line_size = {}
        self.column_line_colour = {}
        self.row_line_size = {}
        self.row_line_colour = {}

        self.parentFrame = parentFrame
        self.SetPreviewSize()

        self.printData = wxPrintData()
        self.scale = 1.0

        self.SetParms()
        self.SetColors()
        self.SetFonts()
        self.TextSpacing()

        self.SetPrinterOffset()
        self.SetHeaderValue()
        self.SetFooterValue()
        self.SetMargins()
        self.SetPortrait()

    def SetPreviewSize(self, position = wxPoint(0, 0), size="Full"):
        if size == "Full":
            r = wxGetClientDisplayRect()
            self.preview_frame_size = wxSize(r.width, r.height)
            self.preview_frame_pos = wxPoint(r.x, r.y)
        else:
            self.preview_frame_size = size
            self.preview_frame_pos = position

    def SetPaperId(self, paper):
        self.printData.SetPaperId(paper)

    def SetOrientation(self, orient):
        self.printData.SetOrientation(orient)

    def SetColors(self):
        self.row_def_line_colour = wxNamedColour('BLACK')
        self.row_def_line_size = 1

        self.column_def_line_colour = wxNamedColour('BLACK')
        self.column_def_line_size = 1
        self.column_colour = wxNamedColour('WHITE')

        self.label_colour = wxNamedColour('LIGHT GREY')

    def SetFonts(self):
        self.label_font = { "Name": self.default_font_name, "Size": 12, "Colour": [0, 0, 0], "Attr": [0, 0, 0] }
        self.text_font = { "Name": self.default_font_name, "Size": 10, "Colour": [0, 0, 0], "Attr": [0, 0, 0] }

    def TextSpacing(self):
        self.label_pt_adj_before = 0     # point adjustment before and after the label text
        self.label_pt_adj_after = 0

        self.text_pt_adj_before = 0     # point adjustment before and after the row text
        self.text_pt_adj_after = 0

    def SetLabelSpacing(self, before, after):        # method to set the label space adjustment
        self.label_pt_adj_before = before
        self.label_pt_adj_after = after

    def SetRowSpacing(self, before, after):         # method to set the row space adjustment
        self.text_pt_adj_before = before
        self.text_pt_adj_after = after

    def SetPrinterOffset(self):        # offset to adjust for printer
        self.vertical_offset = -0.1
        self.horizontal_offset = -0.1

    def SetHeaderValue(self):
        self.header_margin = 0.25
        self.header_font = { "Name": self.default_font_name, "Size": 11, "Colour": [0, 0, 0], "Attr": [0, 0, 0] }
        self.header_align = wxALIGN_CENTRE
        self.header_indent = 0
        self.header_type = "Text"

    def SetFooterValue(self):
        self.footer_margin = 0.7
        self.footer_font = { "Name": self.default_font_name, "Size": 11, "Colour": [0, 0, 0], "Attr": [0, 0, 0] }
        self.footer_align = wxALIGN_CENTRE
        self.footer_indent = 0
        self.footer_type = "Pageof"

    def SetMargins(self):
        self.left_margin = 1.0
        self.right_margin = 1.0     # only used if no column sizes

        self.top_margin  = 0.8
        self.bottom_margin = 1.0
        self.cell_left_margin = 0.1
        self.cell_right_margin = 0.1

    def SetPortrait(self):
        self.printData.SetPaperId(wxPAPER_LETTER)
        self.printData.SetOrientation(wxPORTRAIT)
        self.page_width = 8.5
        self.page_height = 11.0

    def SetLandscape(self):
        self.printData.SetOrientation(wxLANDSCAPE)
        self.page_width = 11.0
        self.page_height = 8.5

    def SetParms(self):
        self.ymax = 1
        self.xmax = 1
        self.page = 1
        self.total_pg = 100

        self.preview = None
        self.page = 0

        self.default_font_name = "Arial"
        self.default_font = { "Name": self.default_font_name, "Size": 10, "Colour": [0, 0, 0], "Attr": [0, 0, 0] }

    def SetColAlignment(self, col, align=wxALIGN_LEFT):
        self.set_column_align[col] = align

    def SetColBackgroundColour(self, col, colour):
        self.set_column_bgcolour[col] = colour

    def SetColTextColour(self, col, colour):
        self.set_column_txtcolour[col] = colour

    def SetCellColour(self, row, col, colour):      # cell background colour
        try:
            set = self.set_cell_colour[row]     # test if row already exists
            try:
                set[col] = colour       # test if column already exists
            except:
                set = { col: colour }       # create the column value
        except:
            set = { col: colour }           # create the column value

        self.set_cell_colour[row] = set    # create dictionary item for colour settings

    def SetCellText(self, row, col, colour):        # font colour for custom cells
        try:
            set = self.set_cell_text[row]     # test if row already exists
            try:
                set[col] = colour       # test if column already exists
            except:
                set = { col: colour }       # create the column value
        except:
            set = { col: colour }           # create the column value

        self.set_cell_text[row] = set    # create dictionary item for colour settings

    def SetColumnLineSize(self, col, size):      # column line size
        self.column_line_size[col] = size    # create dictionary item for column line settings

    def SetColumnLineColour(self, col, colour):
        self.column_line_colour[col] = colour

    def SetRowLineSize(self, row, size):
        self.row_line_size[row] = size

    def SetRowLineColour(self, row, colour):
        self.row_line_colour[row] = colour

    def GetColour(self, colour):        # returns colours based from wxColour value
        red = colour.Red()
        blue = colour.Blue()
        green = colour.Green()
        return [red, green, blue ]

    def SetHeader(self, text = "", type = "Text", font=None, align = None, indent = None, colour = None, size = None):
        set = { "Text": text }

        if font is None:
            set["Font"] = copy.copy(self.default_font)
        else:
            set["Font"] = font

        if colour is not None:
            setfont = set["Font"]
            setfont["Colour"] = self.GetColour(colour)

        if size is not None:
            setfont = set["Font"]
            setfont["Size"] = size

        if align is None:
            set["Align"] = self.header_align
        else:
            set["Align"] = align

        if indent is None:
            set["Indent"] = self.header_indent
        else:
            set["Indent"] = indent

        if type is None:
            set["Type"] = self.header_type
        else:
            set["Type"] = type

        self.header.append(set)

    def SetFooter(self, text = "", type = None, font=None, align = None, indent = None, colour = None, size = None):
        set = { "Text": text }

        if font is None:
            set["Font"] = copy.copy(self.default_font)
        else:
            set["Font"] = font

        if colour is not None:
            setfont = set["Font"]
            setfont["Colour"] = self.GetColour(colour)

        if size is not None:
            setfont = set["Font"]
            setfont["Size"] = size

        if align is None:
            set["Align"] = self.footer_align
        else:
            set["Align"] = align

        if indent is None:
            set["Indent"] = self.footer_indent
        else:
            set["Indent"] = indent

        if type is None:
            set["Type"] = self.footer_type
        else:
            set["Type"] = type

        self.footer.append(set)

    def Preview(self):
        printout = SetPrintout(self)
        printout2 = SetPrintout(self)
        self.preview = wxPrintPreview(printout, printout2, self.printData)
        if not self.preview.Ok():
            wxMessageBox("There was a problem printing!", "Printing", wxOK)
            return

        self.preview.SetZoom(60)        # initial zoom value

        frame = wxPreviewFrame(self.preview, self.parentFrame, "Print preview")

        frame.Initialize()
        if self.parentFrame:
            frame.SetPosition(self.preview_frame_pos)
            frame.SetSize(self.preview_frame_size)
        frame.Show(True)

    def Print(self):
        pdd = wxPrintDialogData()
        pdd.SetPrintData(self.printData)
        printer = wxPrinter(pdd)
        printout = SetPrintout(self)
        if not printer.Print(self.parentFrame, printout):
            wxMessageBox("There was a problem printing.\nPerhaps your current printer is not set correctly?", "Printing", wxOK)
        else:
            self.printData = printer.GetPrintDialogData().GetPrintData()
        printout.Destroy()

    def DoDrawing(self, DC):
        size = DC.GetSizeTuple()
        DC.BeginDrawing()

        table = PrintTableDraw(self, DC, size)
        table.data = self.data
        table.set_column = self.set_column
        table.label = self.label
        table.SetPage(self.page)

        if self.preview is None:
            table.SetPSize(size[0]/self.page_width, size[1]/self.page_height)
            table.SetPTSize(size[0], size[1])
            table.SetPreview(False)
        else:
            if self.preview == 1:
                table.scale = self.scale
                table.SetPSize(size[0]/self.page_width, size[1]/self.page_height)
            else:
                table.SetPSize(self.pwidth, self.pheight)

            table.SetPTSize(self.ptwidth, self.ptheight)
            table.SetPreview(self.preview)

        table.OutCanvas()
        self.page_total = table.total_pages     # total display pages

        DC.EndDrawing()

        self.ymax = DC.MaxY()
        self.xmax = DC.MaxX()

        self.sizeh = size[0]
        self.sizew = size[1]

    def GetTotalPages(self):
        self.page_total = 100
        return self.page_total

    def HasPage(self, page):
        if page <= self.page_total:
            return True
        else:
            return False

    def SetPage(self, page):
        self.page = page

    def SetPageSize(self, width, height):
        self.pwidth, self.pheight = width, height

    def SetTotalSize(self, width, height):
        self.ptwidth, self.ptheight = width, height

    def SetPreview(self, preview, scale):
        self.preview = preview
        self.scale = scale

    def SetTotalSize(self, width, height):
        self.ptwidth = width
        self.ptheight = height

class PrintGrid:
    def __init__(self, parent, grid, format = [], total_col = None, total_row = None):
        if total_row is None:
            total_row = grid.GetNumberRows()
        if total_col is None:
            total_col = grid.GetNumberCols()

        self.total_row = total_row
        self.total_col = total_col
        self.grid = grid

        data = []
        for row in range(total_row):
            row_val = []
            value = grid.GetRowLabelValue(row)
            row_val.append(value)

            for col in range(total_col):
                value = grid.GetCellValue(row, col)
                row_val.append(value)
            data.append(row_val)

        label = [""]
        for col in range(total_col):
            value = grid.GetColLabelValue(col)
            label.append(value)

        self.table = PrintTable(parent)
        self.table.cell_left_margin = 0.0
        self.table.cell_right_margin = 0.0

        self.table.label = label
        self.table.set_column = format
        self.table.data = data

    def GetTable(self):
        return self.table

    def SetAttributes(self):
        for row in range(self.total_row):
            for col in range(self.total_col):
                colour = self.grid.GetCellTextColour(row, col-1)
                self.table.SetCellText(row, col, colour)

                colour = self.grid.GetCellBackgroundColour(row, col-1)
                self.table.SetCellColour(row, col, colour)

    def Preview(self):
        self.table.Preview()

    def Print(self):
        self.table.Print()


class SetPrintout(wxPrintout):
    def __init__(self, canvas):
        wxPrintout.__init__(self)
        self.canvas = canvas
        self.end_pg = 1000

    def OnBeginDocument(self, start, end):
        return self.base_OnBeginDocument(start, end)

    def OnEndDocument(self):
        self.base_OnEndDocument()

    def HasPage(self, page):
        try:
            end = self.canvas.HasPage(page)
            return end
        except:
            return True

    def GetPageInfo(self):
        try:
            self.end_pg = self.canvas.GetTotalPages()
        except:
            pass

        end_pg = self.end_pg
        str_pg = 1
        return (str_pg, end_pg, str_pg, end_pg)

    def OnPreparePrinting(self):
        self.base_OnPreparePrinting()

    def OnBeginPrinting(self):
        dc = self.GetDC()

        self.preview = self.IsPreview()
        if (self.preview):
            self.pixelsPerInch = self.GetPPIScreen()
        else:
            self.pixelsPerInch = self.GetPPIPrinter()

        (w, h) = dc.GetSizeTuple()
        scaleX = float(w) / 1000
        scaleY = float(h) / 1000
        self.printUserScale = min(scaleX, scaleY)

        self.base_OnBeginPrinting()

    def GetSize(self):
        self.psizew, self.psizeh = self.GetPPIPrinter()
        return self.psizew, self.psizeh

    def GetTotalSize(self):
        self.ptsizew, self.ptsizeh = self.GetPageSizePixels()
        return self.ptsizew, self.ptsizeh

    def OnPrintPage(self, page):
        dc = self.GetDC()
        (w, h) = dc.GetSizeTuple()
        scaleX = float(w) / 1000
        scaleY = float(h) / 1000
        self.printUserScale = min(scaleX, scaleY)
        dc.SetUserScale(self.printUserScale, self.printUserScale)

        self.preview = self.IsPreview()

        self.canvas.SetPreview(self.preview, self.printUserScale)
        self.canvas.SetPage(page)

        self.ptsizew, self.ptsizeh = self.GetPageSizePixels()
        self.canvas.SetTotalSize(self.ptsizew, self.ptsizeh)

        self.psizew, self.psizeh = self.GetPPIPrinter()
        self.canvas.SetPageSize(self.psizew, self.psizeh)

        self.canvas.DoDrawing(dc)
        return True









"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: Tree.py,v 1.1.2.1 2003/03/11 19:37:11 RD Exp $"
__revision__ = "$Revision: 1.1.2.1 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


from Base import Object
from Controls import Control
import Parameters as wx


class TreeCtrl(Control):
    """"""

    def AddRoot(self):
        """"""
        pass

    def AppendItem(self):
        """"""
        pass

    def AssignImageList(self):
        """"""
        pass

    def AssignStateImageList(self):
        """"""
        pass

    def Collapse(self):
        """"""
        pass

    def CollapseAndReset(self):
        """"""
        pass

    def Create(self):
        """"""
        pass

    def Delete(self):
        """"""
        pass

    def DeleteAllItems(self):
        """"""
        pass

    def DeleteChildren(self):
        """"""
        pass

    def EditLabel(self):
        """"""
        pass

    def EnsureVisible(self):
        """"""
        pass

    def Expand(self):
        """"""
        pass

    def GetBoundingRect(self):
        """"""
        pass

    def GetChildrenCount(self):
        """"""
        pass

    def GetCount(self):
        """"""
        pass

    def GetEditControl(self):
        """"""
        pass

    def GetFirstChild(self):
        """"""
        pass

    def GetFirstVisibleItem(self):
        """"""
        pass

    def GetImageList(self):
        """"""
        pass

    def GetIndent(self):
        """"""
        pass

    def GetItemBackgroundColour(self):
        """"""
        pass

    def GetItemData(self):
        """"""
        pass

    def GetItemFont(self):
        """"""
        pass

    def GetItemImage(self):
        """"""
        pass

    def GetItemParent(self):
        """"""
        pass

    def GetItemSelectedImage(self):
        """"""
        pass

    def GetItemText(self):
        """"""
        pass

    def GetItemTextColour(self):
        """"""
        pass

    def GetLastChild(self):
        """"""
        pass

    def GetNextChild(self):
        """"""
        pass

    def GetNextSibling(self):
        """"""
        pass

    def GetNextVisible(self):
        """"""
        pass

    def GetPrevSibling(self):
        """"""
        pass

    def GetPrevVisible(self):
        """"""
        pass

    def GetPyData(self):
        """"""
        pass

    def GetRootItem(self):
        """"""
        pass

    def GetSelection(self):
        """"""
        pass

    def GetSelections(self):
        """"""
        pass

    def GetSpacing(self):
        """"""
        pass

    def GetStateImageList(self):
        """"""
        pass

    def HitTest(self):
        """"""
        pass

    def InsertItem(self):
        """"""
        pass

    def InsertItemBefore(self):
        """"""
        pass

    def IsBold(self):
        """"""
        pass

    def IsExpanded(self):
        """"""
        pass

    def IsSelected(self):
        """"""
        pass

    def IsVisible(self):
        """"""
        pass

    def ItemHasChildren(self):
        """"""
        pass

    def PrependItem(self):
        """"""
        pass

    def ScrollTo(self):
        """"""
        pass

    def SelectItem(self):
        """"""
        pass

    def SetImageList(self):
        """"""
        pass

    def SetIndent(self):
        """"""
        pass

    def SetItemBackgroundColour(self):
        """"""
        pass

    def SetItemBold(self):
        """"""
        pass

    def SetItemData(self):
        """"""
        pass

    def SetItemFont(self):
        """"""
        pass

    def SetItemHasChildren(self):
        """"""
        pass

    def SetItemImage(self):
        """"""
        pass

    def SetItemSelectedImage(self):
        """"""
        pass

    def SetItemText(self):
        """"""
        pass

    def SetItemTextColour(self):
        """"""
        pass

    def SetPyData(self):
        """"""
        pass

    def SetSpacing(self):
        """"""
        pass

    def SetStateImageList(self):
        """"""
        pass

    def SortChildren(self):
        """"""
        pass

    def Toggle(self):
        """"""
        pass

    def Unselect(self):
        """"""
        pass

    def UnselectAll(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass

    def _setCallbackInfo(self):
        """"""
        pass


class TreeItemAttr:
    """"""

    def GetBackgroundColour(self):
        """"""
        pass

    def GetFont(self):
        """"""
        pass

    def GetTextColour(self):
        """"""
        pass

    def HasBackgroundColour(self):
        """"""
        pass

    def HasFont(self):
        """"""
        pass

    def HasTextColour(self):
        """"""
        pass

    def SetBackgroundColour(self):
        """"""
        pass

    def SetFont(self):
        """"""
        pass

    def SetTextColour(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class TreeItemData(Object):
    """"""

    def GetData(self):
        """"""
        pass

    def GetId(self):
        """"""
        pass

    def SetData(self):
        """"""
        pass

    def SetId(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class TreeItemId:
    """"""

    def IsOk(self):
        """"""
        pass

    def Ok(self):
        """"""
        pass

    def __cmp__(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass




"""Decorator utility for documentation and shell scripting."""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: decorator.py,v 1.1.2.2 2003/03/20 23:14:27 RD Exp $"
__revision__ = "$Revision: 1.1.2.2 $"[11:-2]


import inspect

try:
    True
except NameError:
    True = 1==1
    False = 1==0

def decorate(real, decoration):
    """Decorate real module with docstrings from decoration module."""
    realdict = real.__dict__
    for item in decoration.__dict__.values():
        if inspect.isclass(item):
            decorateClass(item, realdict)
        elif inspect.isfunction(item):
            decorateFunction(item, realdict)

def decorateClass(item, realdict):
    classname = item.__name__
    if not classname.startswith('wx'):
        classname = 'wx' + classname
    try:
        wxclass = realdict[classname]
    except:
        print classname
    else:
        if item.__doc__:
            wxclass.__doc__ = item.__doc__
        # Get attributes from only the item's local dictionary!
        for attrname, attr in item.__dict__.items():
            # If the attribute has a docstring, and the wx class has a
            # matching attribute.
            if hasattr(attr, '__doc__') and hasattr(wxclass, attrname):
                if inspect.isfunction(attr):
                    # Class methods are functions.
                    doc = getdoc(attr, drop=True)
                    # Is getattr() okay, or do we want to only look in
                    # the wxclass.__dict__ and wxclassPtr.__dict__?
                    wxattr = getattr(wxclass, attrname)
                    # Our class code may be defined incorrectly, and
                    # the wxattr may not actually be a class method,
                    # but that's okay because the following attempt
                    # will simply fail.
                    try:
                        func = wxattr.im_func
                        func.__doc__ = doc
                    except:
                        pass

def decorateFunction(item, realdict):
    funcname = item.__name__
    if funcname in realdict.keys():
        func = realdict[funcname]
        doc = getdoc(item, drop=False)
        try:
            # Built-in functions have a read-only docstring. :-(
            func.__doc__ = doc
        except:
            # print funcname
            pass

def getdoc(attr, drop=False):
    """Return a docstring for attr, which should be a method."""
    doc = ''
    if attr.__doc__:
        doc = inspect.getdoc(attr).strip()
    name = attr.__name__
    # tip is a string with name(argspec), like: "SetLabel(label)"
    tip = ''
    argspec = apply(inspect.formatargspec, inspect.getargspec(attr))
    # The first parameter to a method is a reference to an instance,
    # usually coded as "self", and is usually passed automatically by
    # Python and therefore we want to drop it.
    temp = argspec.split(',')
    if len(temp) == 1:  # No other arguments.
        argspec = '()'
    elif drop:  # Drop the first argument.
        argspec = '(' + ','.join(temp[1:]).lstrip()
    else:
        argspec = ','.join(temp).lstrip()
    tip = name + argspec
    firstline = doc.split('\n')[0].lstrip()
    if tip != firstline:
        doc = '%s\n\n%s' % (tip, doc)
    return doc



#---------------------------------------------------------------------------
# Name:        wxPython.lib.pubsub
# Purpose:     The Publish/Subscribe framework used by evtmgr.EventManager
#
# Author:      Robb Shecter and Robin Dunn
#
# Created:     12-December-2002
# RCS-ID:      $Id: pubsub.py,v 1.1.2.6 2003/02/26 18:38:14 RD Exp $
# Copyright:   (c) 2002 by Robb Shecter <robb@acm.org>
# Licence:     wxWindows license
#---------------------------------------------------------------------------
"""
This module has classes for implementing the Publish/Subscribe design
pattern.

It's a very flexible PS implementation: The message topics are tuples
of any length, containing any objects (that can be used as hash keys).
A subscriber's topic matches any message topic for which it's a
sublist.

It also has many optimizations to favor time efficiency (ie., run-time
speed).  I did this because I use it to support extreme uses.  For
example, piping every wxWindows mouse event through to multiple
listeners, and expecting the app to have no noticeable slowdown.  This
has made the code somewhat obfuscated, but I've done my best to
document it.

The Server and Message classes are the two that clients interact
with..

This module is compatible with Python 2.1.

Author: Robb Shecter
"""

#---------------------------------------------------------------------------

class Publisher:
    """
    The publish/subscribe server.  This class is a Singleton.
    """
    def __init__(self):
        self.topicDict         = {}
        self.functionDict      = {}
        self.subscribeAllList  = []
        self.messageCount      = 0
        self.deliveryCount     = 0


    #
    # Public API
    #

    def subscribe(self, topic, listener):
        """
        Add the given subscription to the list.  This will
        add an entry recording the fact that the listener wants
        to get messages for (at least) the given topic.  This
        method may be called multiple times for one listener,
        registering it with many topics.  It can also be invoked
        many times for a particular topic, each time with a
        different listener.

        listener: expected to be either a method or function that
        takes zero or one parameters.  (Not counting 'self' in the
        case of methods. If it accepts a parameter, it will be given
        a reference to a Message object.

        topic: will  be converted to a tuple if it isn't one.
        It's a pattern matches any topic that it's a sublist
        of.  For example, this pattern:

          ('sports',)

        would match these:

          ('sports',)
          ('sports', 'baseball')
          ('sports', 'baseball', 'highscores')

        but not these:

          ()
          ('news')
          (12345)
        """
        if not callable(listener):
            raise TypeError('The P/S listener, '+`listener`+', is not callable.')
        aTopic = Topic(topic)

        # Determine now (at registration time) how many parameters
        # the listener expects, and get a reference to a function which
        # calls it correctly at message-send time.
        callableVersion = self.__makeCallable(listener)

        # Add this tuple to a list which is in a dict keyed by
        # the topic's first element.
        self.__addTopicToCorrectList(aTopic, listener, callableVersion)

        # Add to a dict in order to speed-up unsubscribing.
        self.__addFunctionLookup(listener, aTopic)


    def unsubscribe(self, listener):
        """
        Remove the given listener from the registry,
        for all topics that it's associated with.
        """
        if not callable(listener):
            raise TypeError('The P/S listener, '+`listener`+', is not callable.')
        topicList = self.getAssociatedTopics(listener)
        for aTopic in topicList:
            subscriberList = self.__getTopicList(aTopic)
            listToKeep = []
            for subscriber in subscriberList:
                if subscriber[0] != listener:
                    listToKeep.append(subscriber)
            self.__setTopicList(aTopic, listToKeep)
        self.__delFunctionLookup(listener)


    def getAssociatedTopics(self, listener):
        """
        Return a list of topics the given listener is
        registered with.
        """
        return self.functionDict.get(listener, [])


    def sendMessage(self, topic, data=None):
        """
        Relay a message to registered listeners.
        """
        aTopic    = Topic(topic)
        message   = Message(aTopic.items, data)
        topicList = self.__getTopicList(aTopic)

        # Send to the matching topics
        for subscriber in topicList:
            if subscriber[1].matches(aTopic):
                subscriber[2](message)

        # Send to any listeners registered for ALL
        for subscriber in self.subscribeAllList:
            subscriber[2](message)


    #
    # Private methods
    #

    def __makeCallable(self, function):
        """
        Return a function that is what the server
        will actually call.

        This is a time optimization: this removes a test
        for the number of parameters from the inner loop
        of sendMessage().
        """
        parameters = self.__parameterCount(function)
        if parameters == 0:
            # Return a function that calls the listener
            # with no arguments.
            return lambda m, f=function: f()
        elif parameters == 1:
            # Return a function that calls the listener
            # with one argument (which will be the message).
            return lambda m, f=function: f(m)
        else:
            raise TypeError('The publish/subscribe listener, '+`function`+', has wrong parameter count')


    def __parameterCount(self, callableObject):
        """
        Return the effective number of parameters required
        by the callable object.  In other words, the 'self'
        parameter of methods is not counted.
        """
        try:
            # Try to handle this like a method
            return callableObject.im_func.func_code.co_argcount - 1
        except AttributeError:
            pass

        try:
            # Try to handle this like a function
            return callableObject.func_code.co_argcount
        except AttributeError:
            raise 'Cannot determine if this is a method or function: '+str(callableObject)

    def __addFunctionLookup(self, aFunction, aTopic):
        try:
            aList = self.functionDict[aFunction]
        except KeyError:
            aList = []
            self.functionDict[aFunction] = aList
        aList.append(aTopic)


    def __delFunctionLookup(self, aFunction):
        try:
            del self.functionDict[aFunction]
        except KeyError:
            print 'Warning: listener not found. Logic error in PublishSubscribe?', aFunction


    def __addTopicToCorrectList(self, topic, listener, callableVersion):
        if len(topic.items) == 0:
            self.subscribeAllList.append((listener, topic, callableVersion))
        else:
            self.__getTopicList(topic).append((listener, topic, callableVersion))


    def __getTopicList(self, aTopic):
        """
        Return the correct sublist of subscribers based on the
        given topic.
        """
        try:
            elementZero = aTopic.items[0]
        except IndexError:
            return self.subscribeAllList

        try:
            subList = self.topicDict[elementZero]
        except KeyError:
            subList = []
            self.topicDict[elementZero] = subList
        return subList


    def __setTopicList(self, aTopic, aSubscriberList):
        try:
            self.topicDict[aTopic.items[0]] = aSubscriberList
        except IndexError:
            self.subscribeAllList = aSubscriberList


    def __call__(self):
        return self


# Create an instance with the same name as the class, effectivly
# hiding the class object so it can't be instantiated any more.  From
# this point forward any calls to Publisher() will invoke the __call__
# of this instance which just returns itself.
#
# The only flaw with this approach is that you can't derive a new
# class from Publisher without jumping through hoops.  If this ever
# becomes an issue then a new Singleton implementaion will need to be
# employed.
Publisher = Publisher()


#---------------------------------------------------------------------------

class Message:
    """
    A simple container object for the two components of
    a message; the topic and the data.
    """
    def __init__(self, topic, data):
        self.topic = topic
        self.data  = data

    def __str__(self):
        return '[Topic: '+`self.topic`+',  Data: '+`self.data`+']'


#---------------------------------------------------------------------------

class Topic:
    """
    A class that represents a publish/subscribe topic.
    Currently, it's only used internally in the framework; the
    API expects and returns plain old tuples.

    It currently exists mostly as a place to keep the matches()
    function.  This function, though, could also correctly be
    seen as an attribute of the P/S server.  Getting rid of this
    class would also mean one fewer object instantiation per
    message send.
    """

    listType   = type([])
    tupleType  = type(())

    def __init__(self, items):
        # Make sure we have a tuple.
        if type(items) == self.__class__.listType:
            items = tuple(items)
        elif type(items) != self.__class__.tupleType:
            items = (items,)
        self.items  = items
        self.length = len(items)


    def matches(self, aTopic):
        """
        Consider myself to be a topic pattern,
        and return True if I match the given specific
        topic.  For example,
        a = ('sports')
        b = ('sports','baseball')
        a.matches(b) --> 1
        b.matches(a) --> 0
        """
        # The question this method answers is equivalent to;
        # is my list a sublist of aTopic's?  So, my algorithm
        # is: 1) make a copy of the aTopic list which is
        # truncated to the pattern's length. 2) Test for
        # equality.
        #
        # This algorithm may be somewhat memory-intensive,
        # because it creates a temporary list on each
        # call to match.  A possible to-do would be to
        # re-write this with a hand-coded loop.
        return (self.items == aTopic.items[:self.length])


    def __repr__(self):
        import string
        return '<Topic>' + string.join(map(repr, self.items), ', ') + '</Topic>'


    def __eq__(self, aTopic):
        """
        Return True if I equal the given topic.  We're considered
        equal if our tuples are equal.
        """
        if type(self) != type(aTopic):
            return 0
        else:
            return self.items == aTopic.items


    def __ne__(self, aTopic):
        """
        Return False if I equal the given topic.
        """
        return not self == aTopic


#---------------------------------------------------------------------------


#
# Code for a simple command-line test
#
if __name__ == '__main__':

    class SimpleListener:
        def __init__(self, number):
            self.number = number
        def notify(self, message):
            print '#'+str(self.number)+' got the message:', message

    # Build a list of ten listeners.
    lList = []
    for x in range(10):
        lList.append(SimpleListener(x))

    server = Publisher()

    # Everyone's interested in politics...
    for x in lList:
        Publisher().subscribe(topic='politics', listener=x.notify)  # also tests singleton

    # But only the first four are interested in trivia.
    for x in lList[:4]:
        server.subscribe(topic='trivia',   listener=x.notify)

    # This one subscribes to everything.
    everythingListener = SimpleListener(999)
    server.subscribe(topic=(), listener=everythingListener.notify)

    # Now send out two messages, testing topic matching.
    server.sendMessage(topic='trivia',               data='What is the capitol of Oregon?')
    server.sendMessage(topic=('politics','germany'), data='The Greens have picked up another seat in the Bundestag.')

#---------------------------------------------------------------------------



"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: Accelerators.py,v 1.1.2.1 2003/03/11 19:37:13 RD Exp $"
__revision__ = "$Revision: 1.1.2.1 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


from Base import Object
import Parameters as wx


class AcceleratorEntry:
    """"""

    def __init__(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def GetCommand(self):
        """"""
        pass

    def GetFlags(self):
        """"""
        pass

    def GetKeyCode(self):
        """"""
        pass

    def Set(self):
        """"""
        pass


class AcceleratorTable(Object):
    """"""

    def __init__(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass





#----------------------------------------------------------------------------
# Name:         wxTimeCtrl.py
# Author:       Will Sadkin
# Created:      09/19/2002
# Copyright:    (c) 2002 by Will Sadkin, 2002
# RCS-ID:       $Id: timectrl.py,v 1.1.2.12 2003/03/11 19:45:41 RD Exp $
# License:      wxWindows license
#----------------------------------------------------------------------------
# NOTE:
#   This was written way it is because of the lack of masked edit controls
#   in wxWindows/wxPython.  I would also have preferred to derive this
#   control from a wxSpinCtrl rather than wxTextCtrl, but the wxTextCtrl
#   component of that control is inaccessible through the interface exposed in
#   wxPython.
#
#   wxTimeCtrl does not use validators, because it does careful manipulation
#   of the cursor in the text window on each keystroke, and validation is
#   cursor-position specific, so the control intercepts the key codes before the
#   validator would fire.
#

from wxPython.wx import *
import string

# wxWindows' wxTextCtrl translates Composite "control key"
# events into single events before returning them to its OnChar
# routine.  The doc says that this results in 1 for Ctrl-A, 2 for
# Ctrl-B, etc. However, there are no wxPython or wxWindows
# symbols for them, so I'm defining codes for Ctrl-X (cut) and
# Ctrl-V (paste) here for readability:
WXK_CTRL_X = (ord('X')+1) - ord('A')
WXK_CTRL_V = (ord('V')+1) - ord('A')

# The following bit of function is for debugging the subsequent code.
# To turn on debugging output, set _debug to 1
_debug = 0
_indent = 0

if _debug:
    def _dbg(*args, **kwargs):
        global _indent

        if len(args):
            if _indent:      print ' ' * 3 * _indent,
            for arg in args: print arg,
            print
        # else do nothing

        # post process args:
        for kwarg, value in kwargs.items():
            if kwarg == 'indent' and value:         _indent = _indent + 1
            elif kwarg == 'indent' and value == 0:  _indent = _indent - 1
            if _indent < 0: _indent = 0
else:
    def _dbg(*args, **kwargs):
        pass


# This class of event fires whenever the value of the time changes in the control:
wxEVT_TIMEVAL_UPDATED = wxNewId()
def EVT_TIMEUPDATE(win, id, func):
    """Used to trap events indicating that the current time has been changed."""
    win.Connect(id, -1, wxEVT_TIMEVAL_UPDATED, func)

class TimeUpdatedEvent(wxPyCommandEvent):
    def __init__(self, id, value ='12:00:00 AM'):
        wxPyCommandEvent.__init__(self, wxEVT_TIMEVAL_UPDATED, id)
        self.value = value
    def GetValue(self):
        """Retrieve the value of the time control at the time this event was generated"""
        return self.value


# Set up all the positions of the cells in the wxTimeCtrl (once at module import):
# Format of control is:
#               hh:mm:ss xM
#                         1
# positions:    01234567890
_listCells = ['hour', 'minute', 'second', 'am_pm']
_listCellRange =   [(0,1,2), (3,4,5), (6,7,8), (9,10,11)]
_listDelimPos =  [2,5,8]

# Create dictionary of cell ranges, indexed by name or position in the range:
_dictCellRange = {}
for i in range(4):
    _dictCellRange[_listCells[i]] = _listCellRange[i]
for cell in _listCells:
    for i in _dictCellRange[cell]:
        _dictCellRange[i] = _dictCellRange[cell]


# Create lists of starting and ending positions for each range, and a dictionary of starting
# positions indexed by name
_listStartCellPos = []
_listEndCellPos = []
for tup in _listCellRange:
    _listStartCellPos.append(tup[0])  # 1st char of cell
    _listEndCellPos.append(tup[1])    # last char of cell (not including delimiter)

_dictStartCellPos = {}
for i in range(4):
    _dictStartCellPos[_listCells[i]] = _listStartCellPos[i]


class wxTimeCtrl(wxTextCtrl):
    def __init__ (
                self, parent, id=-1, value = '12:00:00 AM',
                pos = wxDefaultPosition, size = wxDefaultSize,
                fmt24hr=0,
                spinButton = None,
                style = wxTE_PROCESS_TAB, name = "time"
        ):
        wxTextCtrl.__init__(self, parent, id, value='',
                            pos=pos, size=size, style=style, name=name)

        self.__fmt24hr = fmt24hr

        if size == wxDefaultSize:
            # set appropriate default sizes depending on format:
            if self.__fmt24hr:
                testText = '00:00:00'
            else:
                testText = '00:00:00 MM'
            _dbg(wxPlatform)

            if wxPlatform != "__WXMSW__":   # give it a little extra space
                testText += 'M'
            if wxPlatform == "__WXMAC__":   # give it even a little more...
                testText += 'M'

            w, h = self.GetTextExtent(testText)
            self.SetClientSize( (w+4, self.GetClientSize().height) )


        if self.__fmt24hr:  self.__lastCell = 'second'
        else:               self.__lastCell = 'am_pm'

        # Validate initial value and set if appropriate
        try:
            self.SetValue(value)
        except:
            self.SetValue('12:00:00 AM')

        # set initial position and selection state
        self.__SetCurrentCell(_dictStartCellPos['hour'])
        self.__OnChangePos(None)

        # Set up internal event handlers to change the event reaction behavior of
        # the base wxTextCtrl:
        EVT_TEXT(self, self.GetId(), self.__OnTextChange)
        EVT_SET_FOCUS(self, self.__OnFocus)
        EVT_LEFT_UP(self, self.__OnChangePos)
        EVT_LEFT_DCLICK(self, self.__OnDoubleClick)
        EVT_CHAR(self, self.__OnChar)

        if spinButton:
            self.BindSpinButton(spinButton)     # bind spin button up/down events to this control


    def BindSpinButton(self, sb):
        """
        This function binds an externally created spin button to the control, so that
        up/down events from the button automatically change the control.
        """
        _dbg('wxTimeCtrl::BindSpinButton')
        self.__spinButton = sb
        if self.__spinButton:
            # bind event handlers to spin ctrl
            EVT_SPIN_UP(self.__spinButton, self.__spinButton.GetId(), self.__OnSpinUp)
            EVT_SPIN_DOWN(self.__spinButton, self.__spinButton.GetId(), self.__OnSpinDown)



    def __repr__(self):
        return "<wxTimeCtrl: %s>" % self.GetValue()



    def SetValue(self, value):
        """
        Validating SetValue function for time strings, doing 12/24 format conversion as appropriate.
        """
        _dbg('wxTimeCtrl::SetValue', indent=1)
        dict_range = _dictCellRange             # (for brevity)
        dict_start = _dictStartCellPos

        fmt12len = dict_range['am_pm'][-1]
        fmt24len = dict_range['second'][-1]
        try:
            separators_correct = value[2] == ':' and value[5] == ':'
            len_ok = len(value) in (fmt12len, fmt24len)

            if len(value) > fmt24len:
                separators_correct = separators_correct and value[8] == ' '
            hour = int(value[dict_range['hour'][0]:dict_range['hour'][-1]])
            hour_ok = ((hour in range(0,24) and len(value) == fmt24len)
                       or (hour in range(1,13) and len(value) == fmt12len
                           and value[dict_start['am_pm']:] in ('AM', 'PM')))

            minute = int(value[dict_range['minute'][0]:dict_range['minute'][-1]])
            min_ok  = minute in range(60)
            second  = int(value[dict_range['second'][0]:dict_range['second'][-1]])
            sec_ok  = second in range(60)

            _dbg('len_ok =', len_ok, 'separators_correct =', separators_correct)
            _dbg('hour =', hour, 'hour_ok =', hour_ok, 'min_ok =', min_ok, 'sec_ok =', sec_ok)

            if len_ok and hour_ok and min_ok and sec_ok and separators_correct:
                _dbg('valid time string')

                self.__hour = hour
                if len(value) == fmt12len:                      # handle 12 hour format conversion for actual hour:
                    am = value[dict_start['am_pm']:] == 'AM'
                    if hour != 12 and not am:
                        self.__hour = hour = (hour+12) % 24
                    elif hour == 12:
                        if am: self.__hour = hour = 0

                self.__minute = minute
                self.__second = second

                # valid time
                need_to_convert = ((self.__fmt24hr and len(value) == fmt12len)
                                   or (not self.__fmt24hr and len(value) == fmt24len))
                _dbg('need_to_convert =', need_to_convert)

                if need_to_convert:     #convert to 12/24 hour format as specified:
                    if self.__fmt24hr and len(value) == fmt12len:
                        text = '%.2d:%.2d:%.2d' % (hour, minute, second)
                    else:
                        if hour > 12:
                            hour = hour - 12
                            am_pm = 'PM'
                        elif hour == 12:
                            am_pm = 'PM'
                        else:
                            if hour == 0: hour = 12
                            am_pm = 'AM'
                        text = '%2d:%.2d:%.2d %s' % (hour, minute, second, am_pm)
                else:
                    text = value
                _dbg('text=', text)
                wxTextCtrl.SetValue(self, text)
                _dbg('firing TimeUpdatedEvent...')
                evt = TimeUpdatedEvent(self.GetId(), text)
                evt.SetEventObject(self)
                self.GetEventHandler().ProcessEvent(evt)
            else:
                _dbg('len_ok:', len_ok, 'separators_correct =', separators_correct)
                _dbg('hour_ok:', hour_ok, 'min_ok:', min_ok, 'sec_ok:', sec_ok, indent=0)
                raise ValueError, 'value is not a valid time string'

        except (TypeError, ValueError):
            _dbg(indent=0)
            raise ValueError, 'value is not a valid time string'
        _dbg(indent=0)

    def SetWxDateTime(self, wxdt):
        value = '%2d:%.2d:%.2d' % (wxdt.GetHour(), wxdt.GetMinute(), wxdt.GetSecond())
        self.SetValue(value)

    def GetWxDateTime(self):
        t = wxDateTimeFromHMS(self.__hour, self.__minute, self.__second)
        return t

    def SetMxDateTime(self, mxdt):
        from mx import DateTime
        value = '%2d:%.2d:%.2d' % (mxdt.hour, mxdt.minute, mxdt.second)
        self.SetValue(value)

    def GetMxDateTime(self):
        from mx import DateTime
        t = DateTime.Time(self.__hour, self.__minute, self.__second)
        return t

#-------------------------------------------------------------------------------------------------------------
# these are private functions and overrides:

    def __SetCurrentCell(self, pos):
        """
        Sets state variables that indicate the current cell and position within the control.
        """
        self.__posCurrent = pos
        self.__cellStart, self.__cellEnd = _dictCellRange[pos][0], _dictCellRange[pos][-1]


    def SetInsertionPoint(self, pos):
        """
        Records the specified position and associated cell before calling base class' function.
        """
        _dbg('wxTimeCtrl::SetInsertionPoint', pos, indent=1)

        # Adjust pos to legal value if not already
        if pos < 0: pos = 0
        elif pos in _listDelimPos + [_dictCellRange[self.__lastCell]]:
            pos = pos - 1
            if self.__lastCell == 'am_pm' and pos in _dictCellRange[self.__lastCell]:
                pos = _dictStartCellPos[self.__lastCell]

        self.__SetCurrentCell(pos)
        wxTextCtrl.SetInsertionPoint(self, pos)                 # (causes EVT_TEXT event to fire)
        _dbg(indent=0)


    def SetSelection(self, sel_start, sel_to):
        _dbg('wxTimeCtrl::SetSelection', sel_start, sel_to, indent=1)

        # Adjust selection range to legal extent if not already
        if sel_start < 0:
            self.SetInsertionPoint(0)
            sel_start = self.__posCurrent

        elif sel_start in _listDelimPos + [_dictCellRange[self.__lastCell]]:
            self.SetInsertionPoint(sel_start - 1)
            sel_start = self.__posCurrent

        if self.__posCurrent != sel_start:                      # force selection and insertion point to match
            self.SetInsertionPoint(sel_start)

        if sel_to not in _dictCellRange[sel_start]:
            sel_to = _dictCellRange[sel_start][-1]              # limit selection to end of current cell

        self.__bSelection = sel_start != sel_to
        self.__posSelectTo = sel_to
        wxTextCtrl.SetSelection(self, sel_start, sel_to)
        _dbg(indent=0)


    def __OnFocus(self,event):
        """
        This event handler is currently necessary to work around new default
        behavior as of wxPython2.3.3;
        The TAB key auto selects the entire contents of the wxTextCtrl *after*
        the EVT_SET_FOCUS event occurs; therefore we can't query/adjust the selection
        *here*, because it hasn't happened yet.  So to prevent this behavior, and
        preserve the correct selection when the focus event is not due to tab,
        we need to pull the following trick:
        """
        _dbg('wxTimeCtrl::OnFocus')
        wxCallAfter(self.__FixSelection)
        event.Skip()


    def __FixSelection(self):
        """
        This gets called after the TAB traversal selection is made, if the
        focus event was due to this, but before the EVT_LEFT_* events if
        the focus shift was due to a mouse event.

        The trouble is that, a priori, there's no explicit notification of
        why the focus event we received.  However, the whole reason we need to
        do this is because the default behavior on TAB traveral in a wxTextCtrl is
        now to select the entire contents of the window, something we don't want.
        So we can *now* test the selection range, and if it's "the whole text"
        we can assume the cause, change the insertion point to the start of
        the control, and deselect.
        """
        _dbg('wxTimeCtrl::FixSelection', indent=1)
        sel_start, sel_to = self.GetSelection()
        if sel_start == 0 and sel_to in _dictCellRange[self.__lastCell]:
            # This isn't normally allowed, and so assume we got here by the new
            # "tab traversal" behavior, so we need to reset the selection
            # and insertion point:
            _dbg('entire text selected; resetting selection to start of control')
            self.SetInsertionPoint(0)
            self.SetSelection(self.__cellStart, self.__cellEnd)
        _dbg(indent=0)


    def __OnTextChange(self, event):
        """
        This private event handler is required to retain the current position information of the cursor
        after update to the underlying text control is done.
        """
        _dbg('wxTimeCtrl::OnTextChange', indent=1)
        self.__SetCurrentCell(self.__posCurrent)                # ensure cell range vars are set

        # Note: must call self.SetSelection here to preserve insertion point cursor after update!
        # (I don't know why, but this does the trick!)
        if self.__bSelection:
            _dbg('reselecting from ', self.__posCurrent, 'to', self.__posSelectTo)
            self.SetSelection(self.__posCurrent, self.__posSelectTo)
        else:
            self.SetSelection(self.__posCurrent, self.__posCurrent)
        event.Skip()
        _dbg(indent=0)

    def __OnSpin(self, key):
        self.__IncrementValue(key, self.__posCurrent)
        self.SetFocus()
        self.SetInsertionPoint(self.__posCurrent)
        self.SetSelection(self.__posCurrent, self.__posSelectTo)

    def __OnSpinUp(self, event):
        """
        Event handler for any bound spin button on EVT_SPIN_UP;
        causes control to behave as if up arrow was pressed.
        """
        _dbg('wxTimeCtrl::OnSpinUp', indent=1)
        self.__OnSpin(WXK_UP)
        _dbg(indent=0)


    def __OnSpinDown(self, event):
        """
        Event handler for any bound spin button on EVT_SPIN_DOWN;
        causes control to behave as if down arrow was pressed.
        """
        _dbg('wxTimeCtrl::OnSpinDown', indent=1)
        self.__OnSpin(WXK_DOWN)
        _dbg(indent=0)


    def __OnChangePos(self, event):
        """
        Event handler for motion events; this handler
        changes limits the selection to the new cell boundaries.
        """
        _dbg('wxTimeCtrl::OnChangePos', indent=1)
        pos = self.GetInsertionPoint()
        self.__SetCurrentCell(pos)
        sel_start, sel_to = self.GetSelection()
        selection = sel_start != sel_to
        if not selection:
            # disallow position at end of field:
            if pos in _listDelimPos + [_dictCellRange[self.__lastCell][-1]]:
                self.SetInsertionPoint(pos-1)
            self.__posSelectTo = self.__cellEnd
        else:
            # only allow selection to end of current cell:
            if sel_to < pos:   self.__posSelectTo = self.__cellStart
            elif sel_to > pos: self.__posSelectTo = self.__cellEnd

        _dbg('new pos =', self.__posCurrent, 'select to ', self.__posSelectTo)
        self.SetSelection(self.__posCurrent, self.__posSelectTo)
        if event: event.Skip()
        _dbg(indent=0)


    def __OnDoubleClick(self, event):
        """
        Event handler for left double-click mouse events; this handler
        causes the cell at the double-click point to be selected.
        """
        _dbg('wxTimeCtrl::OnDoubleClick', indent=1)
        pos = self.GetInsertionPoint()
        self.__SetCurrentCell(pos)
        if self.__posCurrent != self.__cellStart:
            self.SetInsertionPoint(self.__cellStart)
        self.SetSelection(self.__cellStart, self.__cellEnd)
        _dbg(indent=0)


    def __OnChar(self, event):
        """
        This private event handler is the main control point for the wxTimeCtrl.
        It governs whether input characters are accepted and if so, handles them
        so as to provide appropriate cursor and selection behavior for the control.
        """
        _dbg('wxTimeCtrl::OnChar', indent=1)

        # NOTE: Returning without calling event.Skip() eats the event before it
        # gets to the text control...

        key = event.GetKeyCode()
        text = self.GetValue()
        pos = self.GetInsertionPoint()
        if pos != self.__posCurrent:
            _dbg("insertion point has moved; resetting current cell")
            self.__SetCurrentCell(pos)
            self.SetSelection(self.__posCurrent, self.__posCurrent)

        sel_start, sel_to = self.GetSelection()
        selection = sel_start != sel_to
        _dbg('sel_start=', sel_start, 'sel_to =', sel_to)
        if not selection:
            self.__bSelection = False                       # predict unselection of entire region

        _dbg('keycode = ', key)
        _dbg('pos = ', pos)

        # don't allow deletion, cut or paste:
        if key in (WXK_DELETE, WXK_BACK, WXK_CTRL_X, WXK_CTRL_V):
            pass

        elif key == WXK_TAB:                                # skip to next field if applicable:
            _dbg('key == WXK_TAB')
            dict_range = _dictCellRange                     # (for brevity)
            dict_start = _dictStartCellPos
            if event.ShiftDown():                           # tabbing backwords

                ###(NOTE: doesn't work; wxTE_PROCESS_TAB doesn't appear to send us this event!)

                _dbg('event.ShiftDown()')
                if pos in dict_range['hour']:               # already in 1st field
                    self.__SetCurrentCell(dict_start['hour']) # ensure we have our member vars set
                    event.Skip()                            #then do normal tab processing for the form
                    _dbg(indent=0)
                    return

                elif pos in dict_range['minute']:           # skip to hours field
                    new_pos = dict_start['hour']
                elif pos in dict_range['second']:           # skip to minutes field
                    new_pos = dict_start['minute']
                elif pos in dict_range['am_pm']:            # skip to seconds field
                    new_pos = dict_start['second']

                self.SetInsertionPoint(new_pos)             # force insert point to jump to next cell (swallowing TAB)
                self.__OnChangePos(None)                    # update selection accordingly

            else:
                # Tabbing forwards through control...

                if pos in dict_range[self.__lastCell]:      # already in last field; ensure we have our members set
                    self.__SetCurrentCell(dict_start[self.__lastCell])
                    _dbg('tab in last cell')
                    event.Skip()                            # then do normal tab processing for the form
                    _dbg(indent=0)
                    return

                if pos in dict_range['second']:             # skip to AM/PM field (if not last cell)
                    new_pos = dict_start['am_pm']
                elif pos in dict_range['minute']:           # skip to seconds field
                    new_pos = dict_start['second']
                elif pos in dict_range['hour']:             # skip to minutes field
                    new_pos = dict_start['minute']

                self.SetInsertionPoint(new_pos)             # force insert point to jump to next cell (swallowing TAB)
                self.__OnChangePos(None)                    # update selection accordingly

        elif key == WXK_LEFT:                               # move left; set insertion point as appropriate:
            _dbg('key == WXK_LEFT')
            if event.ShiftDown():                           # selecting a range...
                _dbg('event.ShiftDown()')
                if pos in _listStartCellPos:                # can't select pass delimiters
                    if( sel_to == pos+2 and sel_to != _dictCellRange['am_pm'][-1]):
                        self.SetSelection(pos, pos+1)       # allow deselection of 2nd char in cell if not am/pm
                    # else ignore event

                elif pos in _listEndCellPos:                # can't use normal selection, because position ends up
                                                            # at delimeter
                    _dbg('set selection from', pos-1, 'to', self.__posCurrent)
                    self.SetInsertionPoint(pos-1)           # this selects the previous position
                    self.SetSelection(self.__posCurrent, pos)
                else:
                    self.SetInsertionPoint(sel_to - 1)      # this unselects the last digit
                    self.SetSelection(self.__posCurrent, pos)

            else:   # ... not selecting
                if pos == 0:                                # can't position before position 0
                    pass
                elif pos in _listStartCellPos:              # skip (left) OVER the colon/space:
                    self.SetInsertionPoint(pos-2)
                    self.__OnChangePos(None)                # set the selection appropriately
                else:
                    self.SetInsertionPoint(pos-1)           # reposition the cursor and
                    self.__OnChangePos(None)                # set the selection appropriately


        elif key == WXK_RIGHT:                              # move right
            _dbg('key == WXK_RIGHT')
            if event.ShiftDown():
                _dbg('event.ShiftDown()')
                if sel_to in _listDelimPos:                 # can't select pass delimiters
                    pass
                else:
                    self.SetSelection(self.__posCurrent, sel_to+1)
            else:
                if( (self.__lastCell == 'second'
                     and pos == _dictStartCellPos['second']+1)
                    or (self.__lastCell == 'am_pm'
                        and pos == _dictStartCellPos['am_pm']) ):
                    pass                                    # don't allow cursor past last cell
                elif pos in _listEndCellPos:                # skip (right) OVER the colon/space:
                    self.SetInsertionPoint(pos+2)
                    self.__OnChangePos(None)                # set the selection appropriately
                else:
                    self.SetInsertionPoint(pos+1)           # reposition the cursor and
                    self.__OnChangePos(None)                # set the selection appropriately

        elif key in (WXK_UP, WXK_DOWN):
            _dbg('key in (WXK_UP, WXK_DOWN)')
            self.__IncrementValue(key, pos)                   # increment/decrement as appropriate


        elif key < WXK_SPACE or key == WXK_DELETE or key > 255:
            event.Skip()                                    # non alphanumeric; process normally (Right thing to do?)

        elif chr(key) in ['!', 'c', 'C']:                   # Special character; sets the value of the control to "now"
            _dbg("key == '!'; setting time to 'now'")
            now = wxDateTime_Now()
            self.SetWxDateTime(now)

        elif chr(key) in string.digits:                     # let ChangeValue validate and update current position
            self.__ChangeValue(chr(key), pos)               # handle event (and swallow it)

        elif chr(key) in ('a', 'A', 'p', 'P', ' '):         # let ChangeValue validate and update current position
            self.__ChangeValue(chr(key), pos)               # handle event (and swallow it)

        else:                                               # disallowed char; swallow event
            pass
        _dbg(indent=0)


    def __IncrementValue(self, key, pos):
        _dbg('wxTimeCtrl::IncrementValue', key, pos, indent=1)
        text = self.GetValue()

        sel_start, sel_to = self.GetSelection()
        selection = sel_start != sel_to
        cell_selected = selection and sel_to -1 != pos

        dict_start = _dictStartCellPos                      # (for brevity)

        # Determine whether we should change the entire cell or just a portion of it:
        if( cell_selected
            or (pos in _listStartCellPos and not selection)
            or (text[pos] == ' ' and text[pos+1] not in ('1', '2'))
            or (text[pos] == '9' and text[pos-1] == ' ' and key == WXK_UP)
            or (text[pos] == '1' and text[pos-1] == ' ' and key == WXK_DOWN)
            or pos >= dict_start['am_pm']):

            self.__IncrementCell(key, pos)
        else:
            if key == WXK_UP:   inc = 1
            else:               inc = -1

            if pos == dict_start['hour'] and not self.__fmt24hr:
                if text[pos] == ' ': digit = '1'                    # allow ' ' or 1 for 1st digit in 12hr format
                else:                digit = ' '
            else:
                if pos == dict_start['hour']:
                    if int(text[pos + 1]) >3:   mod = 2             # allow for 20-23
                    else:                       mod = 3             # allow 00-19
                elif pos == dict_start['hour'] + 1:
                    if self.__fmt24hr:
                        if text[pos - 1] == '2': mod = 4            # allow hours 20-23
                        else:                    mod = 10           # allow hours 00-19
                    else:
                        if text[pos - 1] == '1': mod = 3            # allow hours 10-12
                        else:                    mod = 10           # allow 0-9

                elif pos in (dict_start['minute'],
                             dict_start['second']): mod = 6         # allow minutes/seconds 00-59
                else:                               mod = 10

                digit = '%d' % ((int(text[pos]) + inc) % mod)

            _dbg("new digit = \'%s\'" % digit)
            self.__ChangeValue(digit, pos)
        _dbg(indent=0)


    def __IncrementCell(self, key, pos):
        _dbg('wxTimeCtrl::IncrementCell', key, pos, indent=1)
        self.__SetCurrentCell(pos)                                  # determine current cell
        hour, minute, second = self.__hour, self.__minute, self.__second
        text = self.GetValue()
        dict_start = _dictStartCellPos                              # (for brevity)
        if key == WXK_UP:   inc = 1
        else:               inc = -1

        if self.__cellStart == dict_start['am_pm']:
            am = text[dict_start['am_pm']:] == 'AM'
            if am: hour = hour + 12
            else:  hour = hour - 12
        else:
            if self.__cellStart == dict_start['hour']:
                hour = (hour + inc) % 24
            elif self.__cellStart == dict_start['minute']:
                minute = (minute + inc) % 60
            elif self.__cellStart == dict_start['second']:
                second = (second + inc) % 60

        newvalue = '%.2d:%.2d:%.2d' % (hour, minute, second)

        self.SetValue(newvalue)
        self.SetInsertionPoint(self.__cellStart)
        self.SetSelection(self.__cellStart, self.__cellEnd)
        _dbg(indent=0)


    def __ChangeValue(self, char, pos):
        _dbg('wxTimeCtrl::ChangeValue', "\'" + char + "\'", pos, indent=1)
        text = self.GetValue()

        self.__SetCurrentCell(pos)
        sel_start, sel_to = self.GetSelection()
        self.__posSelectTo = sel_to
        self.__bSelection = selection = sel_start != sel_to
        cell_selected = selection and sel_to -1 != pos
        _dbg('cell_selected =', cell_selected, indent=0)

        dict_start = _dictStartCellPos                          # (for brevity)

        if pos in _listDelimPos: return                         # don't allow change of punctuation

        elif( 0 < pos < dict_start['am_pm'] and char not in string.digits):
            return                                              # AM/PM not allowed in this position

        # See if we're changing the hour cell, and validate/update appropriately:
        #
        hour_start = dict_start['hour']                         # (ie. 0)

        if pos == hour_start:                                   # if at 1st position,
            if self.__fmt24hr:                                  # and using 24 hour format
                if cell_selected:                               # replace cell contents with hour represented by digit
                    newtext = '%.2d' % int(char) + text[hour_start+2:]
                elif char not in ('0', '1', '2'):               # return if digit not 0,1, or 2
                    return
                else:                                           # relace current position
                    newtext = char + text[pos+1:]
            else:                                               # (12 hour format)
                if cell_selected:
                    if char == ' ': return                      # can't erase entire cell
                    elif char == '0':                           # treat 0 as '12'
                        newtext = '12' + text[hour_start+2:]
                    else:                                       # replace cell contents with hour represented by digit
                        newtext = '%2d' % int(char) + text[hour_start+2:]
                else:
                    if char not in ('1', ' '):                  # can only type a 1 or space
                        return
                    if text[pos+1] not in ('0', '1', '2'):      # and then, only if other column is 0,1, or 2
                        return
                    if char == ' ' and text[pos+1] == '0':      # and char isn't space if 2nd column is 0
                        return
                    else:                                       # ok; replace current position
                        newtext = char + text[pos+1:]
                if char == ' ': self.SetInsertionPoint(pos+1)   # move insert point to legal position

        elif pos == hour_start+1:                               # if editing 2nd position of hour
            if( not self.__fmt24hr                              # and using 12 hour format
                and text[hour_start] == '1'                     # if 1st char is 1,
                and char not in ('0', '1', '2')):               # disallow anything bug 0,1, or 2
                return
            newtext = text[hour_start] + char + text[hour_start+2:]  # else any digit ok

        # Do the same sort of validation for minute and second cells
        elif pos in (dict_start['minute'], dict_start['second']):
            if cell_selected:                                   # if cell selected, replace value
                newtext = text[:pos] + '%.2d' % int(char) + text[pos+2:]
            elif int(char) > 5: return                          # else disallow > 59 for minute and second fields
            else:
                newtext = text[:pos] + char + text[pos+1:]      # else ok

        elif pos in (dict_start['minute']+1, dict_start['second']+1):
            newtext = text[:pos] + char + text[pos+1:]          # all digits ok for 2nd digit of minute/second

        # Process AM/PM cell
        elif pos == dict_start['am_pm']:
            char = char.upper()
            if char not in ('A','P'): return                    # disallow all but A or P as 1st char of column
            newtext = text[:pos] + char + text[pos+1:]
        else: return    # not a valid position

        _dbg(indent=1)
        # update member position vars and set selection to character changed
        if not cell_selected:
            _dbg('reselecting current digit')
            self.__posSelectTo = pos+1

        _dbg('newtext=', newtext)
        self.SetValue(newtext)
        self.SetInsertionPoint(self.__posCurrent)
        self.SetSelection(self.__posCurrent, self.__posSelectTo)
        _dbg(indent=0)


    def Cut(self):
        """
        Override wxTextCtrl::Cut() method, as this operation should not
        be allowed for wxTimeCtrls.
        """
        return


    def Paste(self):
        """
        Override wxTextCtrl::Paste() method, as this operation should not
        be allowed for wxTimeCtrls.
        """
        return


#----------------------------------------------------------------------------
# Test jig for wxTimeCtrl:

if __name__ == '__main__':
    import traceback

    class TestPanel(wxPanel):
        def __init__(self, parent, id,
                     pos = wxPyDefaultPosition, size = wxPyDefaultSize,
                     fmt24hr = 0, test_mx = 0,
                     style = wxTAB_TRAVERSAL ):

            wxPanel.__init__(self, parent, id, pos, size, style)

            self.test_mx = test_mx

            self.tc = wxTimeCtrl(self, 10, fmt24hr = fmt24hr)
            sb = wxSpinButton( self, 20, wxDefaultPosition, wxSize(-1,20), 0 )
            self.tc.BindSpinButton(sb)

            sizer = wxBoxSizer( wxHORIZONTAL )
            sizer.AddWindow( self.tc, 0, wxALIGN_CENTRE|wxLEFT|wxTOP|wxBOTTOM, 5 )
            sizer.AddWindow( sb, 0, wxALIGN_CENTRE|wxRIGHT|wxTOP|wxBOTTOM, 5 )

            self.SetAutoLayout( True )
            self.SetSizer( sizer )
            sizer.Fit( self )
            sizer.SetSizeHints( self )

            EVT_TIMEUPDATE(self, self.tc.GetId(), self.OnTimeChange)

        def OnTimeChange(self, event):
            _dbg('OnTimeChange: value = ', event.GetValue())
            wxdt = self.tc.GetWxDateTime()
            _dbg('wxdt =', wxdt.GetHour(), wxdt.GetMinute(), wxdt.GetSecond())
            if self.test_mx:
                mxdt = self.tc.GetMxDateTime()
                _dbg('mxdt =', mxdt.hour, mxdt.minute, mxdt.second)


    class MyApp(wxApp):
        def OnInit(self):
            import sys
            fmt24hr = '24' in sys.argv
            test_mx = 'mx' in sys.argv
            try:
                frame = wxFrame(NULL, -1, "wxTimeCtrl Test", wxPoint(20,20), wxSize(100,100) )
                panel = TestPanel(frame, -1, wxPoint(-1,-1), fmt24hr=fmt24hr, test_mx = test_mx)
                frame.Show(True)
            except:
                traceback.print_exc()
                return False
            return True

    try:
        app = MyApp(0)
        app.MainLoop()
    except:
        traceback.print_exc()



# This file was created automatically by SWIG.
import imagec

from misc import *

from gdi import *

from fonts import *

from streams import *
class wxImageHandlerPtr(wxObjectPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def GetName(self, *_args, **_kwargs):
        val = apply(imagec.wxImageHandler_GetName,(self,) + _args, _kwargs)
        return val
    def GetExtension(self, *_args, **_kwargs):
        val = apply(imagec.wxImageHandler_GetExtension,(self,) + _args, _kwargs)
        return val
    def GetType(self, *_args, **_kwargs):
        val = apply(imagec.wxImageHandler_GetType,(self,) + _args, _kwargs)
        return val
    def GetMimeType(self, *_args, **_kwargs):
        val = apply(imagec.wxImageHandler_GetMimeType,(self,) + _args, _kwargs)
        return val
    def CanRead(self, *_args, **_kwargs):
        val = apply(imagec.wxImageHandler_CanRead,(self,) + _args, _kwargs)
        return val
    def SetName(self, *_args, **_kwargs):
        val = apply(imagec.wxImageHandler_SetName,(self,) + _args, _kwargs)
        return val
    def SetExtension(self, *_args, **_kwargs):
        val = apply(imagec.wxImageHandler_SetExtension,(self,) + _args, _kwargs)
        return val
    def SetType(self, *_args, **_kwargs):
        val = apply(imagec.wxImageHandler_SetType,(self,) + _args, _kwargs)
        return val
    def SetMimeType(self, *_args, **_kwargs):
        val = apply(imagec.wxImageHandler_SetMimeType,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxImageHandler instance at %s>" % (self.this,)
class wxImageHandler(wxImageHandlerPtr):
    def __init__(self,this):
        self.this = this




class wxPNGHandlerPtr(wxImageHandlerPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __repr__(self):
        return "<C wxPNGHandler instance at %s>" % (self.this,)
class wxPNGHandler(wxPNGHandlerPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(imagec.new_wxPNGHandler,_args,_kwargs)
        self.thisown = 1




class wxJPEGHandlerPtr(wxImageHandlerPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __repr__(self):
        return "<C wxJPEGHandler instance at %s>" % (self.this,)
class wxJPEGHandler(wxJPEGHandlerPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(imagec.new_wxJPEGHandler,_args,_kwargs)
        self.thisown = 1




class wxBMPHandlerPtr(wxImageHandlerPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __repr__(self):
        return "<C wxBMPHandler instance at %s>" % (self.this,)
class wxBMPHandler(wxBMPHandlerPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(imagec.new_wxBMPHandler,_args,_kwargs)
        self.thisown = 1




class wxICOHandlerPtr(wxBMPHandlerPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __repr__(self):
        return "<C wxICOHandler instance at %s>" % (self.this,)
class wxICOHandler(wxICOHandlerPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(imagec.new_wxICOHandler,_args,_kwargs)
        self.thisown = 1




class wxCURHandlerPtr(wxICOHandlerPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __repr__(self):
        return "<C wxCURHandler instance at %s>" % (self.this,)
class wxCURHandler(wxCURHandlerPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(imagec.new_wxCURHandler,_args,_kwargs)
        self.thisown = 1




class wxANIHandlerPtr(wxCURHandlerPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __repr__(self):
        return "<C wxANIHandler instance at %s>" % (self.this,)
class wxANIHandler(wxANIHandlerPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(imagec.new_wxANIHandler,_args,_kwargs)
        self.thisown = 1




class wxGIFHandlerPtr(wxImageHandlerPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __repr__(self):
        return "<C wxGIFHandler instance at %s>" % (self.this,)
class wxGIFHandler(wxGIFHandlerPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(imagec.new_wxGIFHandler,_args,_kwargs)
        self.thisown = 1




class wxPNMHandlerPtr(wxImageHandlerPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __repr__(self):
        return "<C wxPNMHandler instance at %s>" % (self.this,)
class wxPNMHandler(wxPNMHandlerPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(imagec.new_wxPNMHandler,_args,_kwargs)
        self.thisown = 1




class wxPCXHandlerPtr(wxImageHandlerPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __repr__(self):
        return "<C wxPCXHandler instance at %s>" % (self.this,)
class wxPCXHandler(wxPCXHandlerPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(imagec.new_wxPCXHandler,_args,_kwargs)
        self.thisown = 1




class wxTIFFHandlerPtr(wxImageHandlerPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __repr__(self):
        return "<C wxTIFFHandler instance at %s>" % (self.this,)
class wxTIFFHandler(wxTIFFHandlerPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(imagec.new_wxTIFFHandler,_args,_kwargs)
        self.thisown = 1




class wxImagePtr(wxObjectPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=imagec.delete_wxImage):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def Create(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_Create,(self,) + _args, _kwargs)
        return val
    def Destroy(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_Destroy,(self,) + _args, _kwargs)
        return val
    def Scale(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_Scale,(self,) + _args, _kwargs)
        if val: val = wxImagePtr(val) ; val.thisown = 1
        return val
    def Rescale(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_Rescale,(self,) + _args, _kwargs)
        if val: val = wxImagePtr(val) 
        return val
    def SetRGB(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_SetRGB,(self,) + _args, _kwargs)
        return val
    def GetRed(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetRed,(self,) + _args, _kwargs)
        return val
    def GetGreen(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetGreen,(self,) + _args, _kwargs)
        return val
    def GetBlue(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetBlue,(self,) + _args, _kwargs)
        return val
    def FindFirstUnusedColour(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_FindFirstUnusedColour,(self,) + _args, _kwargs)
        return val
    def SetMaskFromImage(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_SetMaskFromImage,(self,) + _args, _kwargs)
        return val
    def LoadFile(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_LoadFile,(self,) + _args, _kwargs)
        return val
    def LoadMimeFile(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_LoadMimeFile,(self,) + _args, _kwargs)
        return val
    def SaveFile(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_SaveFile,(self,) + _args, _kwargs)
        return val
    def SaveMimeFile(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_SaveMimeFile,(self,) + _args, _kwargs)
        return val
    def LoadStream(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_LoadStream,(self,) + _args, _kwargs)
        return val
    def LoadMimeStream(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_LoadMimeStream,(self,) + _args, _kwargs)
        return val
    def Ok(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_Ok,(self,) + _args, _kwargs)
        return val
    def GetWidth(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetWidth,(self,) + _args, _kwargs)
        return val
    def GetHeight(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetHeight,(self,) + _args, _kwargs)
        return val
    def GetSubImage(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetSubImage,(self,) + _args, _kwargs)
        if val: val = wxImagePtr(val) ; val.thisown = 1
        return val
    def Copy(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_Copy,(self,) + _args, _kwargs)
        if val: val = wxImagePtr(val) ; val.thisown = 1
        return val
    def Paste(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_Paste,(self,) + _args, _kwargs)
        return val
    def GetDataBuffer(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetDataBuffer,(self,) + _args, _kwargs)
        return val
    def GetData(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetData,(self,) + _args, _kwargs)
        return val
    def SetDataBuffer(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_SetDataBuffer,(self,) + _args, _kwargs)
        return val
    def SetData(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_SetData,(self,) + _args, _kwargs)
        return val
    def SetMaskColour(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_SetMaskColour,(self,) + _args, _kwargs)
        return val
    def GetMaskRed(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetMaskRed,(self,) + _args, _kwargs)
        return val
    def GetMaskGreen(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetMaskGreen,(self,) + _args, _kwargs)
        return val
    def GetMaskBlue(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetMaskBlue,(self,) + _args, _kwargs)
        return val
    def SetMask(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_SetMask,(self,) + _args, _kwargs)
        return val
    def HasMask(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_HasMask,(self,) + _args, _kwargs)
        return val
    def Rotate(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_Rotate,(self,) + _args, _kwargs)
        if val: val = wxImagePtr(val) ; val.thisown = 1
        return val
    def Rotate90(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_Rotate90,(self,) + _args, _kwargs)
        if val: val = wxImagePtr(val) ; val.thisown = 1
        return val
    def Mirror(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_Mirror,(self,) + _args, _kwargs)
        if val: val = wxImagePtr(val) ; val.thisown = 1
        return val
    def Replace(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_Replace,(self,) + _args, _kwargs)
        return val
    def ConvertToMono(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_ConvertToMono,(self,) + _args, _kwargs)
        if val: val = wxImagePtr(val) ; val.thisown = 1
        return val
    def SetOption(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_SetOption,(self,) + _args, _kwargs)
        return val
    def SetOptionInt(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_SetOptionInt,(self,) + _args, _kwargs)
        return val
    def GetOption(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetOption,(self,) + _args, _kwargs)
        return val
    def GetOptionInt(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_GetOptionInt,(self,) + _args, _kwargs)
        return val
    def HasOption(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_HasOption,(self,) + _args, _kwargs)
        return val
    def CountColours(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_CountColours,(self,) + _args, _kwargs)
        return val
    def ConvertToBitmap(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_ConvertToBitmap,(self,) + _args, _kwargs)
        if val: val = wxBitmapPtr(val) ; val.thisown = 1
        return val
    def ConvertToMonoBitmap(self, *_args, **_kwargs):
        val = apply(imagec.wxImage_ConvertToMonoBitmap,(self,) + _args, _kwargs)
        if val: val = wxBitmapPtr(val) ; val.thisown = 1
        return val
    def __repr__(self):
        return "<C wxImage instance at %s>" % (self.this,)
class wxImage(wxImagePtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(imagec.new_wxImage,_args,_kwargs)
        self.thisown = 1






#-------------- FUNCTION WRAPPERS ------------------

def wxEmptyImage(*_args, **_kwargs):
    val = apply(imagec.wxEmptyImage,_args,_kwargs)
    if val: val = wxImagePtr(val); val.thisown = 1
    return val

def wxImageFromMime(*_args, **_kwargs):
    val = apply(imagec.wxImageFromMime,_args,_kwargs)
    if val: val = wxImagePtr(val); val.thisown = 1
    return val

def wxImageFromBitmap(*_args, **_kwargs):
    val = apply(imagec.wxImageFromBitmap,_args,_kwargs)
    if val: val = wxImagePtr(val); val.thisown = 1
    return val

def wxImageFromData(*_args, **_kwargs):
    val = apply(imagec.wxImageFromData,_args,_kwargs)
    if val: val = wxImagePtr(val); val.thisown = 1
    return val

def wxImageFromStream(*_args, **_kwargs):
    val = apply(imagec.wxImageFromStream,_args,_kwargs)
    if val: val = wxImagePtr(val); val.thisown = 1
    return val

def wxImageFromStreamMime(*_args, **_kwargs):
    val = apply(imagec.wxImageFromStreamMime,_args,_kwargs)
    if val: val = wxImagePtr(val); val.thisown = 1
    return val

wxInitAllImageHandlers = imagec.wxInitAllImageHandlers

def wxBitmapFromImage(*_args, **_kwargs):
    val = apply(imagec.wxBitmapFromImage,_args,_kwargs)
    if val: val = wxBitmapPtr(val); val.thisown = 1
    return val

wxImage_CanRead = imagec.wxImage_CanRead

wxImage_GetImageCount = imagec.wxImage_GetImageCount

wxImage_CanReadStream = imagec.wxImage_CanReadStream

wxImage_AddHandler = imagec.wxImage_AddHandler

wxImage_InsertHandler = imagec.wxImage_InsertHandler

wxImage_RemoveHandler = imagec.wxImage_RemoveHandler



#-------------- VARIABLE WRAPPERS ------------------

cvar = imagec.cvar
wxNullImage = wxImagePtr(imagec.cvar.wxNullImage)



"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: Process.py,v 1.1.2.1 2003/03/11 19:37:12 RD Exp $"
__revision__ = "$Revision: 1.1.2.1 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


from Base import EvtHandler


class Process(EvtHandler):
    """"""

    def CloseOutput(self):
        """"""
        pass

    def Destroy(self):
        """"""
        pass

    def Detach(self):
        """"""
        pass

    def GetErrorStream(self):
        """"""
        pass

    def GetInputStream(self):
        """"""
        pass

    def GetOutputStream(self):
        """"""
        pass

    def IsErrorAvailable(self):
        """"""
        pass

    def IsInputAvailable(self):
        """"""
        pass

    def IsInputOpened(self):
        """"""
        pass

    def IsRedirected(self):
        """"""
        pass

    def Redirect(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass

    def _setCallbackInfo(self):
        """"""
        pass

    def base_OnTerminate(self):
        """"""
        pass





#
#  This was modified from rpcMixin.py distributed with wxPython
#
#----------------------------------------------------------------------
# Name:        rpcMixin
# Version:     0.2.0
# Purpose:     provides xmlrpc server functionality for wxPython
#              applications via a mixin class
#
# Requires:    (1) Python with threading enabled.
#              (2) xmlrpclib from PythonWare
#                  (http://www.pythonware.com/products/xmlrpc/)
#                  the code was developed and tested using version 0.9.8
#
# Author:      greg Landrum (Landrum@RationalDiscovery.com)
#
# Copyright:   (c) 2000, 2001 by Greg Landrum and Rational Discovery LLC
# Licence:     wxWindows license
#----------------------------------------------------------------------

"""provides xmlrpc server functionality for wxPython applications via a mixin class

**Some Notes:**

  1)  The xmlrpc server runs in a separate thread from the main GUI
      application, communication between the two threads using a custom
      event (see the Threads demo in the wxPython docs for more info).

  2)  Neither the server nor the client are particularly smart about
      checking method names.  So it's easy to shoot yourself in the foot
      by calling improper methods.  It would be pretty easy to add
      either a list of allowed methods or a list of forbidden methods.

  3)  Authentication of xmlrpc clients is *not* performed.  I think it
      would be pretty easy to do this in a hacky way, but I haven't done
      it yet.

  4)  See the bottom of this file for an example of using the class.

**Obligatory disclaimer:**
  This is my first crack at both using xmlrpc and multi-threaded
  programming, so there could be huge horrible bugs or design
  flaws. If you see one, I'd love to hear about them.

"""


""" ChangeLog
23 May 2001:  Version bumped to 0.2.0
  Numerous code and design changes

21 Mar. 2001:  Version bumped to 0.1.4
  Updated rpcMixin.OnExternal to support methods with further references
   (i.e. now you can do rpcClient.foo.bar() and have it work)
  This probably ain't super legal in xmlrpc land, but it works just fine here
   and we need it.

6  Mar. 2001:  Version bumped to 0.1.3
  Documentation changes to make this compatible with happydoc

21 Jan. 2001:  Version bumped to 0.1.2
  OnExternal() method in the mixin class now uses getattr() to check if
    a desired method is present.  It should have been done this way in
    the first place.
14 Dec. 2000:  Version bumped to 0.1.1
  rearranged locking code and made other changes so that multiple
    servers in one application are possible.

"""

from wxPython.wx import *
import xmlrpcserver,xmlrpclib
import threading
import SocketServer
import new
import sys

rpcPENDING = 0
rpcDONE = 1
rpcEXCEPT = 2

class RPCRequest:
  """A wrapper to use for handling requests and their responses"""
  status = rpcPENDING
  result = None

# here's the ID for external events
wxEVT_EXTERNAL_EVENT = 25015
class ExternalEvent(wxPyEvent):
  """The custom event class used to pass xmlrpc calls from
     the server thread into the GUI thread

  """
  def __init__(self,method,args):
    wxPyEvent.__init__(self)
    self.SetEventType(wxEVT_EXTERNAL_EVENT)
    self.method = method
    self.args = args
    self.rpcStatus = RPCRequest()
    self.rpcStatusLock = threading.Lock()
    self.rpcCondVar = threading.Condition()

  def Destroy(self):
    self.method=None
    self.args=None
    self.rpcStatus = None
    self.rpcStatusLock = None
    self.rpcondVar = None

def EVT_EXTERNAL_EVENT(win,func):
  win.Connect(-1,-1,wxEVT_EXTERNAL_EVENT,func)

class Handler(xmlrpcserver.RequestHandler):
  """The handler class that the xmlrpcserver actually calls
     when a request comes in.

  """
  def log_message(self,*args):
    """ causes the server to stop spewing messages every time a request comes in

    """
    pass
  def call(self,method,params):
    """When an xmlrpc request comes in, this is the method that
       gets called.

       **Arguments**

         - method: name of the method to be called

         - params: arguments to that method

    """
    if method == '_rpcPing':
      # we just acknowledge these without processing them
      return 'ack'

    # construct the event
    evt = ExternalEvent(method,params)

    # update the status variable
    evt.rpcStatusLock.acquire()
    evt.rpcStatus.status = rpcPENDING
    evt.rpcStatusLock.release()

    evt.rpcCondVar.acquire()
    # dispatch the event to the GUI
    wxPostEvent(self._app,evt)

    # wait for the GUI to finish
    while evt.rpcStatus.status == rpcPENDING:
      evt.rpcCondVar.wait()
    evt.rpcCondVar.release()
    evt.rpcStatusLock.acquire()
    if evt.rpcStatus.status == rpcEXCEPT:
      # The GUI threw an exception, release the status lock
      #  and re-raise the exception
      evt.rpcStatusLock.release()
      raise evt.rpcStatus.result[0],evt.rpcStatus.result[1]
    else:
      # everything went through without problems
      s = evt.rpcStatus.result

      evt.rpcStatusLock.release()
      evt.Destroy()
      self._app = None
      return s

# this global Event is used to let the server thread
#  know when it should quit
stopEvent = threading.Event()
stopEvent.clear()

class _ServerThread(threading.Thread):
  """ this is the Thread class which actually runs the server

  """
  def __init__(self,server,verbose=0):
    self._xmlServ = server
    threading.Thread.__init__(self,verbose=verbose)

  def stop(self):
    stopEvent.set()

  def shouldStop(self):
    return stopEvent.isSet()

  def run(self):
    while not self.shouldStop():
      self._xmlServ.handle_request()
    self._xmlServ = None

class rpcMixin:
  """A mixin class to provide xmlrpc server functionality to wxPython
     frames/windows

     If you want to customize this, probably the best idea is to
     override the OnExternal method, which is what's invoked when an
     RPC is handled.

  """

  # we'll try a range of ports for the server, this is the size of the
  #  range to be scanned
  nPortsToTry=20
  if sys.platform == 'win32':
    defPort = 800
  else:
    defPort = 8023

  def __init__(self,host='',port=-1,verbose=0,portScan=1):
    """Constructor

      **Arguments**

        - host: (optional) the hostname for the server

        - port: (optional) the port the server will use

        - verbose: (optional) if set, the server thread will be launched
          in verbose mode

        - portScan: (optional) if set, we'll scan across a number of ports
          to find one which is avaiable

    """
    if port == -1:
      port = self.defPort
    self.verbose=verbose
    EVT_EXTERNAL_EVENT(self,self.OnExternal)
    if hasattr(self,'OnClose'):
      self._origOnClose = self.OnClose
      self.Disconnect(-1,-1,wxEVT_CLOSE_WINDOW)
    else:
      self._origOnClose = None
    self.OnClose = self.RPCOnClose
    EVT_CLOSE(self,self.RPCOnClose)

    tClass = new.classobj('Handler%d'%(port),(Handler,),{})
    tClass._app = self
    if portScan:
      self.rpcPort = -1
      for i in xrange(self.nPortsToTry):
        try:
          xmlServ = SocketServer.TCPServer((host,port+i),tClass)
        except:
          pass
        else:
          self.rpcPort = port+i
    else:
      self.rpcPort = port
      try:
        xmlServ = SocketServer.TCPServer((host,port),tClass)
      except:
        self.rpcPort = -1

    if self.rpcPort == -1:
      raise 'RPCMixinError','Cannot initialize server'
    self.servThread = _ServerThread(xmlServ,verbose=self.verbose)
    self.servThread.setName('XML-RPC Server')
    self.servThread.start()

  def RPCOnClose(self,event):
    """ callback for when the application is closed

       be sure to shutdown the server and the server thread before
       leaving

    """
    # by setting the global stopEvent we inform the server thread
    # that it's time to shut down.
    stopEvent.set()
    if event is not None:
      # if we came in here from a user event (as opposed to an RPC event),
      #  then we'll need to kick the server one last time in order
      #  to get that thread to terminate.  do so now
      s1 = xmlrpclib.Server('http://localhost:%d'%(self.rpcPort))
      try:
        s1._rpcPing()
      except:
        pass

    if self._origOnClose is not None:
      self._origOnClose(event)

  def RPCQuit(self):
    """ shuts down everything, including the rpc server

    """
    self.RPCOnClose(None)
  def OnExternal(self,event):
    """ this is the callback used to handle RPCs

      **Arguments**

        - event: an _ExternalEvent_ sent by the rpc server

      Exceptions are caught and returned in the global _rpcStatus
      structure.  This allows the xmlrpc server to report the
      exception to the client without mucking up any of the delicate
      thread stuff.

    """
    event.rpcStatusLock.acquire()
    doQuit = 0
    try:
      methsplit = event.method.split('.')
      meth = self
      for piece in methsplit:
        meth = getattr(meth,piece)
    except AttributeError,msg:
      event.rpcStatus.result = 'No Such Method',msg
      event.rpcStatus.status = rpcEXCEPT
    else:
      try:
        res = apply(meth,event.args)
      except:
        import traceback
        if self.verbose: traceback.print_exc()
        event.rpcStatus.result = sys.exc_info()[:2]
        event.rpcStatus.status = rpcEXCEPT
      else:
        if res is None:
          # returning None across the xmlrpc interface is problematic
          event.rpcStatus.result = []
        else:
          event.rpcStatus.result = res
        event.rpcStatus.status = rpcDONE

    event.rpcStatusLock.release()

    # broadcast (using the condition var) that we're done with the event
    event.rpcCondVar.acquire()
    event.rpcCondVar.notify()
    event.rpcCondVar.release()


if __name__ == '__main__':
  import time
  if sys.platform == 'win32':
    port = 800
  else:
    port = 8023

  class rpcFrame(wxFrame,rpcMixin):
    """A simple wxFrame with the rpcMixin functionality added
    """
    def __init__(self,*args,**kwargs):
      """ rpcHost or rpcPort keyword arguments will be passed along to
          the xmlrpc server.
      """
      mixinArgs = {}
      if kwargs.has_key('rpcHost'):
        mixinArgs['host'] = kwargs['rpcHost']
        del kwargs['rpcHost']
      if kwargs.has_key('rpcPort'):
        mixinArgs['port'] = kwargs['rpcPort']
        del kwargs['rpcPort']
      if kwargs.has_key('rpcPortScan'):
        mixinArgs['portScan'] = kwargs['rpcPortScan']
        del kwargs['rpcPortScan']

      apply(wxFrame.__init__,(self,)+args,kwargs)
      apply(rpcMixin.__init__,(self,),mixinArgs)

      EVT_CHAR(self,self.OnChar)

    def TestFunc(self,args):
      """a demo method"""
      return args

    def OnChar(self,event):
      key = event.GetKeyCode()
      if key == ord('q'):
        self.OnQuit(event)

    def OnQuit(self,event):
      self.OnClose(event)

    def OnClose(self,event):
      self.Destroy()



  class MyApp(wxApp):
    def OnInit(self):
      self.frame = rpcFrame(NULL, -1, "wxPython RPCDemo", wxDefaultPosition,
                            wxSize(300,300),
                            rpcHost='localhost',rpcPort=port)
      self.frame.Show(True)
      return True


  def testcon(port):
    s1 = xmlrpclib.Server('http://localhost:%d'%(port))
    s1.SetTitle('Munged')
    s1._rpcPing()
    if doQuit:
      s1.RPCQuit()

  doQuit = 1
  if len(sys.argv)>1 and sys.argv[1] == '-q':
    doQuit = 0
  nT = threading.activeCount()
  app = MyApp(0)
  activePort = app.frame.rpcPort
  t = threading.Thread(target=lambda x=activePort:testcon(x),verbose=0)
  t.start()

  app.MainLoop()
  # give the threads time to shut down
  if threading.activeCount() > nT:
    print 'waiting for all threads to terminate'
    while threading.activeCount() > nT:
      time.sleep(0.5)





"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: Help.py,v 1.1.2.1 2003/03/11 19:37:12 RD Exp $"
__revision__ = "$Revision: 1.1.2.1 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


from Base import Object
import Parameters as wx
from Window import Window


class PopupWindow(Window):
    """"""

    def Create(self):
        """"""
        pass

    def Position(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class PopupTransientWindow(PopupWindow):
    """"""

    def Dismiss(self):
        """"""
        pass

    def Popup(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass

    def _setCallbackInfo(self):
        """"""
        pass


class TipProvider:
    """"""

    def GetCurrentTip(self):
        """"""
        pass

    def GetTip(self):
        """"""
        pass

    def PreprocessTip(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class PyTipProvider(TipProvider):
    """"""

    def __init__(self):
        """"""
        pass

    def _setCallbackInfo(self):
        """"""
        pass


class TipWindow(PopupTransientWindow):
    """"""

    def Close(self):
        """"""
        pass

    def SetBoundingRect(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class ToolTip(Object):
    """"""

    def GetTip(self):
        """"""
        pass

    def GetWindow(self):
        """"""
        pass

    def SetTip(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass





# This file was created automatically by SWIG.
import calendarc

from misc import *

from windows import *

from gdi import *

from fonts import *

from clip_dnd import *

from controls import *

from events import *

from utils import *
import wx

def EVT_CALENDAR(win, id, fn):
    win.Connect(id, -1, wxEVT_CALENDAR_DOUBLECLICKED, fn)

def EVT_CALENDAR_SEL_CHANGED(win, id, fn):
    win.Connect(id, -1, wxEVT_CALENDAR_SEL_CHANGED, fn)

def EVT_CALENDAR_DAY(win, id, fn):
    win.Connect(id, -1, wxEVT_CALENDAR_DAY_CHANGED, fn)

def EVT_CALENDAR_MONTH(win, id, fn):
    win.Connect(id, -1, wxEVT_CALENDAR_MONTH_CHANGED, fn)

def EVT_CALENDAR_YEAR(win, id, fn):
    win.Connect(id, -1, wxEVT_CALENDAR_YEAR_CHANGED, fn)

def EVT_CALENDAR_WEEKDAY_CLICKED(win, id, fn):
    win.Connect(id, -1, wxEVT_CALENDAR_WEEKDAY_CLICKED, fn)


class wxCalendarDateAttrPtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def SetTextColour(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_SetTextColour,(self,) + _args, _kwargs)
        return val
    def SetBackgroundColour(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_SetBackgroundColour,(self,) + _args, _kwargs)
        return val
    def SetBorderColour(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_SetBorderColour,(self,) + _args, _kwargs)
        return val
    def SetFont(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_SetFont,(self,) + _args, _kwargs)
        return val
    def SetBorder(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_SetBorder,(self,) + _args, _kwargs)
        return val
    def SetHoliday(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_SetHoliday,(self,) + _args, _kwargs)
        return val
    def HasTextColour(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_HasTextColour,(self,) + _args, _kwargs)
        return val
    def HasBackgroundColour(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_HasBackgroundColour,(self,) + _args, _kwargs)
        return val
    def HasBorderColour(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_HasBorderColour,(self,) + _args, _kwargs)
        return val
    def HasFont(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_HasFont,(self,) + _args, _kwargs)
        return val
    def HasBorder(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_HasBorder,(self,) + _args, _kwargs)
        return val
    def IsHoliday(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_IsHoliday,(self,) + _args, _kwargs)
        return val
    def GetTextColour(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_GetTextColour,(self,) + _args, _kwargs)
        if val: val = wxColourPtr(val) ; val.thisown = 1
        return val
    def GetBackgroundColour(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_GetBackgroundColour,(self,) + _args, _kwargs)
        if val: val = wxColourPtr(val) ; val.thisown = 1
        return val
    def GetBorderColour(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_GetBorderColour,(self,) + _args, _kwargs)
        if val: val = wxColourPtr(val) ; val.thisown = 1
        return val
    def GetFont(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_GetFont,(self,) + _args, _kwargs)
        if val: val = wxFontPtr(val) ; val.thisown = 1
        return val
    def GetBorder(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarDateAttr_GetBorder,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxCalendarDateAttr instance at %s>" % (self.this,)
class wxCalendarDateAttr(wxCalendarDateAttrPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(calendarc.new_wxCalendarDateAttr,_args,_kwargs)
        self.thisown = 1



def wxCalendarDateAttrBorder(*_args,**_kwargs):
    val = wxCalendarDateAttrPtr(apply(calendarc.new_wxCalendarDateAttrBorder,_args,_kwargs))
    val.thisown = 1
    return val


class wxCalendarEventPtr(wxCommandEventPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def GetDate(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarEvent_GetDate,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def GetWeekDay(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarEvent_GetWeekDay,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxCalendarEvent instance at %s>" % (self.this,)
class wxCalendarEvent(wxCalendarEventPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(calendarc.new_wxCalendarEvent,_args,_kwargs)
        self.thisown = 1




class wxCalendarCtrlPtr(wxControlPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def Create(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_Create,(self,) + _args, _kwargs)
        return val
    def SetDate(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_SetDate,(self,) + _args, _kwargs)
        return val
    def GetDate(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_GetDate,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetLowerDateLimit(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_SetLowerDateLimit,(self,) + _args, _kwargs)
        return val
    def GetLowerDateLimit(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_GetLowerDateLimit,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetUpperDateLimit(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_SetUpperDateLimit,(self,) + _args, _kwargs)
        return val
    def GetUpperDateLimit(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_GetUpperDateLimit,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetDateRange(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_SetDateRange,(self,) + _args, _kwargs)
        return val
    def EnableYearChange(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_EnableYearChange,(self,) + _args, _kwargs)
        return val
    def EnableMonthChange(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_EnableMonthChange,(self,) + _args, _kwargs)
        return val
    def EnableHolidayDisplay(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_EnableHolidayDisplay,(self,) + _args, _kwargs)
        return val
    def SetHeaderColours(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_SetHeaderColours,(self,) + _args, _kwargs)
        return val
    def GetHeaderColourFg(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_GetHeaderColourFg,(self,) + _args, _kwargs)
        if val: val = wxColourPtr(val) ; val.thisown = 1
        return val
    def GetHeaderColourBg(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_GetHeaderColourBg,(self,) + _args, _kwargs)
        if val: val = wxColourPtr(val) ; val.thisown = 1
        return val
    def SetHighlightColours(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_SetHighlightColours,(self,) + _args, _kwargs)
        return val
    def GetHighlightColourFg(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_GetHighlightColourFg,(self,) + _args, _kwargs)
        if val: val = wxColourPtr(val) ; val.thisown = 1
        return val
    def GetHighlightColourBg(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_GetHighlightColourBg,(self,) + _args, _kwargs)
        if val: val = wxColourPtr(val) ; val.thisown = 1
        return val
    def SetHolidayColours(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_SetHolidayColours,(self,) + _args, _kwargs)
        return val
    def GetHolidayColourFg(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_GetHolidayColourFg,(self,) + _args, _kwargs)
        if val: val = wxColourPtr(val) ; val.thisown = 1
        return val
    def GetHolidayColourBg(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_GetHolidayColourBg,(self,) + _args, _kwargs)
        if val: val = wxColourPtr(val) ; val.thisown = 1
        return val
    def GetAttr(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_GetAttr,(self,) + _args, _kwargs)
        if val: val = wxCalendarDateAttrPtr(val) 
        return val
    def SetAttr(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_SetAttr,(self,) + _args, _kwargs)
        return val
    def SetHoliday(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_SetHoliday,(self,) + _args, _kwargs)
        return val
    def ResetAttr(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_ResetAttr,(self,) + _args, _kwargs)
        return val
    def HitTest(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_HitTest,(self,) + _args, _kwargs)
        return val
    def Enable(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_Enable,(self,) + _args, _kwargs)
        return val
    def Show(self, *_args, **_kwargs):
        val = apply(calendarc.wxCalendarCtrl_Show,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxCalendarCtrl instance at %s>" % (self.this,)
class wxCalendarCtrl(wxCalendarCtrlPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(calendarc.new_wxCalendarCtrl,_args,_kwargs)
        self.thisown = 1
        self._setOORInfo(self)



def wxPreCalendarCtrl(*_args,**_kwargs):
    val = wxCalendarCtrlPtr(apply(calendarc.new_wxPreCalendarCtrl,_args,_kwargs))
    val.thisown = 1
    val._setOORInfo(val)
    return val




#-------------- FUNCTION WRAPPERS ------------------



#-------------- VARIABLE WRAPPERS ------------------

wxCAL_SUNDAY_FIRST = calendarc.wxCAL_SUNDAY_FIRST
wxCAL_MONDAY_FIRST = calendarc.wxCAL_MONDAY_FIRST
wxCAL_SHOW_HOLIDAYS = calendarc.wxCAL_SHOW_HOLIDAYS
wxCAL_NO_YEAR_CHANGE = calendarc.wxCAL_NO_YEAR_CHANGE
wxCAL_NO_MONTH_CHANGE = calendarc.wxCAL_NO_MONTH_CHANGE
wxCAL_SEQUENTIAL_MONTH_SELECTION = calendarc.wxCAL_SEQUENTIAL_MONTH_SELECTION
wxCAL_SHOW_SURROUNDING_WEEKS = calendarc.wxCAL_SHOW_SURROUNDING_WEEKS
wxCAL_HITTEST_NOWHERE = calendarc.wxCAL_HITTEST_NOWHERE
wxCAL_HITTEST_HEADER = calendarc.wxCAL_HITTEST_HEADER
wxCAL_HITTEST_DAY = calendarc.wxCAL_HITTEST_DAY
wxCAL_HITTEST_INCMONTH = calendarc.wxCAL_HITTEST_INCMONTH
wxCAL_HITTEST_DECMONTH = calendarc.wxCAL_HITTEST_DECMONTH
wxCAL_HITTEST_SURROUNDING_WEEK = calendarc.wxCAL_HITTEST_SURROUNDING_WEEK
wxCAL_BORDER_NONE = calendarc.wxCAL_BORDER_NONE
wxCAL_BORDER_SQUARE = calendarc.wxCAL_BORDER_SQUARE
wxCAL_BORDER_ROUND = calendarc.wxCAL_BORDER_ROUND
wxEVT_CALENDAR_DOUBLECLICKED = calendarc.wxEVT_CALENDAR_DOUBLECLICKED
wxEVT_CALENDAR_SEL_CHANGED = calendarc.wxEVT_CALENDAR_SEL_CHANGED
wxEVT_CALENDAR_DAY_CHANGED = calendarc.wxEVT_CALENDAR_DAY_CHANGED
wxEVT_CALENDAR_MONTH_CHANGED = calendarc.wxEVT_CALENDAR_MONTH_CHANGED
wxEVT_CALENDAR_YEAR_CHANGED = calendarc.wxEVT_CALENDAR_YEAR_CHANGED
wxEVT_CALENDAR_WEEKDAY_CLICKED = calendarc.wxEVT_CALENDAR_WEEKDAY_CLICKED


#-------------- USER INCLUDE -----------------------


# Stuff these names into the wx namespace so wxPyConstructObject can find them

wx.wxCalendarEventPtr                  = wxCalendarEventPtr
wx.wxCalendarCtrlPtr                   = wxCalendarCtrlPtr



#----------------------------------------------------------------------
# Name:        wxPython.lib.rightalign
# Purpose:     A class derived from wxTextCtrl that aligns the text
#              on the right side of the control, (except when editing.)
#
# Author:      Josu Oyanguren
#
# Created:     19-October-2001
# RCS-ID:      $Id: rightalign.py,v 1.1 2001/10/21 03:43:58 RD Exp $
# Copyright:   (c) 2001 by Total Control Software
# Licence:     wxWindows license
#----------------------------------------------------------------------

"""
Some time ago, I asked about how to right-align
wxTextCtrls. Answer was that it is not supported. I forgot it.

Just a week ago, one of my clients asked me to have numbers right
aligned. (Indeed it was that numbers MUST be right aligned).

So the game begun. Hacking, hacking, ...

At last, i succeed. Here is some code that someone may find
useful. ubRightTextCtrl is right-aligned when you are not editing, but
left-aligned if it has focus.

Hope this can help someone, as much as this list helps me.

Josu Oyanguren
Ubera Servicios Informáticos.
"""

from wxPython.wx import *

#----------------------------------------------------------------------

class wxRightTextCtrl(wxTextCtrl):
    def __init__(self, parent, id, *args, **kwargs):
        wxTextCtrl.__init__(self, parent, id, *args, **kwargs)
        EVT_KILL_FOCUS(self, self.OnKillFocus)
        EVT_PAINT(self, self.OnPaint)

    def OnPaint(self, event):
        dc = wxPaintDC(self)
        dc.SetFont(self.GetFont())
        dc.Clear()
        text = self.GetValue()
        textwidth, textheight = dc.GetTextExtent(text)
        dcwidth, dcheight = self.GetClientSizeTuple()

        y = (dcheight - textheight) / 2
        x = dcwidth - textwidth - 2

        dc.SetClippingRegion(0, 0, dcwidth, dcheight)
        dc.DrawText(text, x, y)

        if x < 0:
            toofat = '...'
            markwidth = dc.GetTextExtent(toofat)[0]
            dc.SetPen(wxPen(dc.GetBackground().GetColour(), 1, wxSOLID ))
            dc.DrawRectangle(0,0, markwidth, dcheight)
            dc.SetPen(wxPen(wxRED, 1, wxSOLID ))
            dc.SetBrush(wxTRANSPARENT_BRUSH)
            dc.DrawRectangle(1, 1, dcwidth-2, dcheight-2)
            dc.DrawText(toofat, 1, y)


    def OnKillFocus(self, event):
        if not self.GetParent(): return
        self.Refresh()
        event.Skip()




# This file was created automatically by SWIG.
import glcanvasc

from misc import *

from misc2 import *

from windows import *

from gdi import *

from fonts import *

from clip_dnd import *

from events import *

from streams import *

from utils import *

from mdi import *

from frames import *

from stattool import *

from controls import *

from controls2 import *

from windows2 import *

from cmndlgs import *

from windows3 import *

from image import *

from printfw import *

from sizers import *

from filesys import *
import wx
class wxGLContextPtr(wxObjectPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=glcanvasc.delete_wxGLContext):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def SetCurrent(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLContext_SetCurrent,(self,) + _args, _kwargs)
        return val
    def SetColour(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLContext_SetColour,(self,) + _args, _kwargs)
        return val
    def SwapBuffers(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLContext_SwapBuffers,(self,) + _args, _kwargs)
        return val
    def SetupPixelFormat(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLContext_SetupPixelFormat,(self,) + _args, _kwargs)
        return val
    def SetupPalette(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLContext_SetupPalette,(self,) + _args, _kwargs)
        return val
    def CreateDefaultPalette(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLContext_CreateDefaultPalette,(self,) + _args, _kwargs)
        if val: val = wxPalettePtr(val) ; val.thisown = 1
        return val
    def GetPalette(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLContext_GetPalette,(self,) + _args, _kwargs)
        if val: val = wxPalettePtr(val) 
        return val
    def GetWindow(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLContext_GetWindow,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxGLContext instance at %s>" % (self.this,)
class wxGLContext(wxGLContextPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(glcanvasc.new_wxGLContext,_args,_kwargs)
        self.thisown = 1




class wxGLCanvasPtr(wxWindowPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def SetCurrent(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLCanvas_SetCurrent,(self,) + _args, _kwargs)
        return val
    def SetColour(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLCanvas_SetColour,(self,) + _args, _kwargs)
        return val
    def SwapBuffers(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLCanvas_SwapBuffers,(self,) + _args, _kwargs)
        return val
    def GetContext(self, *_args, **_kwargs):
        val = apply(glcanvasc.wxGLCanvas_GetContext,(self,) + _args, _kwargs)
        if val: val = wxGLContextPtr(val) 
        return val
    def __repr__(self):
        return "<C wxGLCanvas instance at %s>" % (self.this,)
class wxGLCanvas(wxGLCanvasPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(glcanvasc.new_wxGLCanvas,_args,_kwargs)
        self.thisown = 1
        self._setOORInfo(self)



def wxGLCanvasWithContext(*_args,**_kwargs):
    val = wxGLCanvasPtr(apply(glcanvasc.new_wxGLCanvasWithContext,_args,_kwargs))
    val.thisown = 1
    val._setOORInfo(self)
    return val




#-------------- FUNCTION WRAPPERS ------------------



#-------------- VARIABLE WRAPPERS ------------------

WX_GL_RGBA = glcanvasc.WX_GL_RGBA
WX_GL_BUFFER_SIZE = glcanvasc.WX_GL_BUFFER_SIZE
WX_GL_LEVEL = glcanvasc.WX_GL_LEVEL
WX_GL_DOUBLEBUFFER = glcanvasc.WX_GL_DOUBLEBUFFER
WX_GL_STEREO = glcanvasc.WX_GL_STEREO
WX_GL_AUX_BUFFERS = glcanvasc.WX_GL_AUX_BUFFERS
WX_GL_MIN_RED = glcanvasc.WX_GL_MIN_RED
WX_GL_MIN_GREEN = glcanvasc.WX_GL_MIN_GREEN
WX_GL_MIN_BLUE = glcanvasc.WX_GL_MIN_BLUE
WX_GL_MIN_ALPHA = glcanvasc.WX_GL_MIN_ALPHA
WX_GL_DEPTH_SIZE = glcanvasc.WX_GL_DEPTH_SIZE
WX_GL_STENCIL_SIZE = glcanvasc.WX_GL_STENCIL_SIZE
WX_GL_MIN_ACCUM_RED = glcanvasc.WX_GL_MIN_ACCUM_RED
WX_GL_MIN_ACCUM_GREEN = glcanvasc.WX_GL_MIN_ACCUM_GREEN
WX_GL_MIN_ACCUM_BLUE = glcanvasc.WX_GL_MIN_ACCUM_BLUE
WX_GL_MIN_ACCUM_ALPHA = glcanvasc.WX_GL_MIN_ACCUM_ALPHA



#----------------------------------------------------------------------
# Name:        wxPython.tools.helpviewer
# Purpose:     HTML Help viewer
#
# Author:      Robin Dunn
#
# Created:     11-Dec-2002
# RCS-ID:      $Id: helpviewer.py,v 1.1.2.4 2003/01/01 05:18:16 RD Exp $
# Copyright:   (c) 2002 by Total Control Software
# Licence:     wxWindows license
#----------------------------------------------------------------------

"""
helpviewer.py   --  Displays HTML Help in a wxHtmlHelpController window.

Usage:
    helpviewer [--cache=path] helpfile [helpfile(s)...]

    Where helpfile is the path to either a .hhp file or a .zip file
    which contians a .hhp file.  The .hhp files are the same as those
    used by Microsoft's HTML Help Workshop for creating CHM files.
"""


import sys, os

#---------------------------------------------------------------------------

def main(args=sys.argv):
    if len(args) < 2:
        print __doc__
        return

    args = args[1:]
    cachedir = None
    if args[0][:7] == '--cache':
        cachedir = os.path.expanduser(args[0].split('=')[1])
        args = args[1:]

    if len(args) == 0:
        print __doc__
        return

    from wxPython.wx import wxPySimpleApp, wxConfigBase_Get, \
         wxLog_SetActiveTarget, wxLogStderr, \
         wxLog_SetLogLevel, wxLOG_Error, \
         wxFileSystem_AddHandler, wxZipFSHandler
    import wxPython.html
    from wxPython.htmlhelp import wxHtmlHelpController


    app = wxPySimpleApp()
    #wxLog_SetActiveTarget(wxLogStderr())
    wxLog_SetLogLevel(wxLOG_Error)

    # Set up the default config so the htmlhelp frame can save its preferences
    app.SetVendorName('wxWindows')
    app.SetAppName('helpviewer')
    cfg = wxConfigBase_Get()

    # Add the Zip filesystem
    wxFileSystem_AddHandler(wxZipFSHandler())

    # Create the viewer
    helpctrl = wxHtmlHelpController()
    if cachedir:
        helpctrl.SetTempDir(cachedir)

    # and add the books
    for helpfile in args:
        print "Adding %s..." % helpfile
        helpctrl.AddBook(helpfile, 1)

    # start it up!
    helpctrl.DisplayContents()
    app.MainLoop()


if __name__ == '__main__':
    main()
